#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time    : 2018/2/28 10:58
# @Email  : jtyoui@qq.com
# @Software: PyCharm
import math
import re
from jtyoui.regular import Non_Chinese
from jtyoui.decorator import replace_regular
from collections import Counter
import os


@replace_regular(Non_Chinese, '')
def clean(data):
    # 去除非中文字符
    return data, len(data)


class Neologism:
    def __init__(self):
        self.ALL_WORDS = dict()
        self.All_LENS = 0

    def read_file(self, file, split_num):
        """按文件读取
        :param file: 文件地址
        :param split_num: 最大分割词语
        """
        if os.path.exists(file):
            with open(file, encoding='utf-8') as fp:
                for line in fp:
                    self.read_string(line, split_num)

    def read_string(self, st, split_num, split_seq='[，。！？：.,?]'):
        """按字符按照split_seq格式来分割
        :param st: 字符串
        :param split_num:分词的个数
        :param split_seq: 字符分割
        :return: None
        """
        ls = re.split(split_seq, st)
        self.read_ls(ls, split_num)

    def read_ls(self, ls, split_num):
        """数据类型[str]
        :param ls: 表示链表
        :param split_num:分词的个数
        """
        for word in ls:
            self.All_LENS += len(word)
            clean_data, lens = clean(data=word)
            if lens > 2:
                self._split(clean_data, lens, split_num)

    def _split(self, words, lens, split_num):
        """拆分字符，最大匹配num个字符，并也字典的形式返回，
        [出现次数,出现频率,凝固程度,自由程度,关键字的左邻,关键字的右邻](作为信息熵的衡量)
        """
        for i in range(lens):
            for j in range(i + 1, split_num + 1 + i):
                if j < lens:
                    key = words[i:j]
                    word = self.ALL_WORDS.get(key)
                    if word:
                        word[0] += 1
                        word[4].append(words[i - 1])
                        word[5].append(words[j])
                    else:
                        self.ALL_WORDS[key] = [1, 0.0, 1, 0, [words[i - 1]], [words[j]]]

    def statistics(self):
        """统计每个单词的频率"""
        for key in self.ALL_WORDS:
            self.ALL_WORDS[key][1] = self.ALL_WORDS[key][0] / self.All_LENS

    def _information_entropy(self, word_ls):
        """信息熵"""
        entropy_all = 0.0
        key_count = Counter(word_ls)
        for key, count in key_count.items():
            word = self.ALL_WORDS.get(key)
            if word:
                entropy_all -= math.log(word[1]) * word[1] * count  # 邻字的信息熵
        return entropy_all

    def handle(self):
        """处理数据
        计算左邻字集合和右邻字集合的频率，左邻字信息熵和右邻字信息熵中的较小值
        计算凝固程度,自由程度
        """
        for key, word_list in self.ALL_WORDS.items():
            if len(key) > 1:
                # 左邻字集合的凝聚度和右邻字集合的凝聚度相比较.谁越少说明该词语越容易接近谁
                left = word_list[1] / (self.ALL_WORDS[key[0]][1] * self.ALL_WORDS[key[1:]][1])  # 左邻字集合的凝聚度
                right = word_list[1] / (self.ALL_WORDS[key[-1]][1] * self.ALL_WORDS[key[:-1]][1])  # 右邻字集合的凝聚度
                word_list[2] = left if left < right else right
                # 左邻字集合的信息熵和右邻字集合的信息熵的相比较.谁的信息熵越少说明该集合提供的信息越大
                front_all = self._information_entropy(word_list[4])  # 左邻字集合的信息熵
                end_all = self._information_entropy(word_list[5])  # 右邻字集合的信息熵
                word_list[3] = front_all if front_all < end_all else end_all

    def filter_words(self, count, frequency, cond, free):
        """过滤一些不重要的数据
        [出现次数,出现频率,凝固程度,自由程度]
        :param count:key出现的次数
        :param frequency: 过滤的频率
        :param cond:过滤凝聚度
        :param free:过滤自由度
        :return:过滤后的数据字典
        """
        ls = []
        for key, one_word in self.ALL_WORDS.items():
            if len(key) <= 1:
                continue
            if (one_word[0] > count or one_word[1] > frequency) and one_word[2] > cond and one_word[3] > free:
                ls.append((key, one_word))
        return remove_subset(ls)


def remove_subset(ls: list) -> list:
    """去除列表中的子集。比如：['aa','a','ab'] --> ['aa','ab']"""
    ls.sort(key=lambda x: len(x[0]), reverse=True)
    total = []
    for subset in ls:
        flag = True
        for word in total:
            if subset[0] in word[0]:
                flag = False
                break
        if flag:
            total.append(subset)
    return total


def mains(file, split_num, count, frequency, cond, free):
    """成词发现算法。保存结果在当前运行环境下的result.txt文件中
    :param file: 文本地址
    :param split_num: 成词最大粒度
    :param count: key出现的次数
    :param frequency: 过滤的频率
    :param cond: 过滤凝聚度
    :param free: 过滤自由度
    """
    wf = open('result.txt', 'w', encoding='utf-8')
    n = Neologism()
    n.read_file(file, split_num)
    n.statistics()
    n.handle()
    print('正在保存文件数据：result.txt')
    for k, v in n.filter_words(count=count, frequency=frequency, cond=cond, free=free):
        s = F'关键字:{k} 次数:{v[0]} 频率:{v[1]} 凝聚度:{v[2]} 自由度:{v[3]}'
        wf.write(s + '\n')
    wf.flush()
    wf.close()
    print('保存完毕')


if __name__ == '__main__':
    mains(r'D:\data.txt', 6, count=10, frequency=0.0001, cond=84, free=0.7)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time    : 2018/1/2 16:30
# @Email  : jtyoui@qq.com
# @Software: PyCharm

import random
import sys
import time
from jtyoui.error import LibraryNotInstallError
from urllib.request import urlretrieve
import os

try:
    import pygame
except ModuleNotFoundError:
    raise LibraryNotInstallError("安装pygame包: pip install pygame")

SCREEN_WIDTH, SCREEN_HEIGHT = 1200, 700
MY_BIRTH_LEFT, MY_BIRTH_TOP = SCREEN_WIDTH / 2, SCREEN_HEIGHT - 60
DIRECTION = [U, D, L, R] = ['U', 'D', 'L', 'R']
Tank_IMAGE_POSITION = r'D:/tank_img'
URL = 'https://gitee.com/tyoui/logo/raw/master/img/'


def load_img(name_img):
    save = Tank_IMAGE_POSITION + os.sep + name_img + '.gif'
    if not os.path.exists(save):
        urlretrieve(URL + name_img + '.gif', save)
    return pygame.image.load(save)


def load_music(name_music):
    save = Tank_IMAGE_POSITION + os.sep + name_music + '.wav'
    if not os.path.exists(save):
        urlretrieve(URL + name_music + '.wav', save)
    pygame.mixer.music.load(save)
    pygame.mixer.music.play()


class TankGame:
    my_bullet_list = list()
    enemy_bullet_list = list()
    enemy_tank_list = list()
    wall_list = list()

    def __init__(self):
        if not os.path.exists(Tank_IMAGE_POSITION):
            os.makedirs(Tank_IMAGE_POSITION)
        pygame.init()
        pygame.font.init()
        self.display = pygame.display
        self.window = self.display.set_mode([SCREEN_WIDTH, SCREEN_HEIGHT], pygame.RESIZABLE, 32)
        self.display.set_caption('坦克世界')
        self.my_tank = MyTank(MY_BIRTH_LEFT, MY_BIRTH_TOP, self.window)
        self.creat_enemy_number = 10
        self.my_tank_lift = 3
        self.creat_enemy(self.creat_enemy_number)
        self.creat_walls()
        self.font = pygame.font.SysFont('kai_ti', 18)
        self.number = 1

    def creat_enemy(self, number):
        for _ in range(number):
            left = random.randint(0, SCREEN_WIDTH - self.my_tank.tank_width)
            enemy_tank = EnemyTank(left, 20, self.window)
            TankGame.enemy_tank_list.append(enemy_tank)

    def creat_walls(self):
        for i in range(SCREEN_WIDTH // 60 + 1):
            wall_h = random.randint(100, 500)
            w = Wall(60 * i, wall_h, self.window)
            TankGame.wall_list.append(w)

    @staticmethod
    def show_walls():
        for w in TankGame.wall_list:
            if w.live:
                w.display_wall()
            else:
                TankGame.wall_list.remove(w)

    def start_game(self):
        load_music('start')
        while True:
            self.window.fill([0, 0, 0])
            self.get_event()
            len_enemy = len(TankGame.enemy_tank_list)
            self.window.blit(
                self.draw_text('敌方坦克*{0},我方生命值*{1},当前{2}关'.format(len_enemy, self.my_tank_lift, self.number)), (10, 10))
            if len_enemy == 0:
                self.creat_enemy_number += 10
                self.number += 1
                self.my_tank_lift += 1
                self.creat_enemy(self.creat_enemy_number)
                self.wall_list.clear()
                self.creat_walls()
            self.show_my_tank()
            self.show_enemy_tank()
            self.show_bullet(TankGame.enemy_bullet_list)
            self.show_bullet(TankGame.my_bullet_list)
            self.show_walls()
            self.display.update()
            time.sleep(0.02)

    def show_my_tank(self):
        if self.my_tank.live:
            self.my_tank.display()
            self.my_tank.tank_collide_tank()
            self.my_tank.tank_collide_wall()
        else:
            Explode(self.my_tank, self.window).display_explode()
            del self.my_tank
            if self.my_tank_lift == 0:
                self.end_game()
            self.my_tank_lift -= 1
            load_music('add')
            self.my_tank = MyTank(MY_BIRTH_LEFT, MY_BIRTH_TOP, self.window)
        if not self.my_tank.move_stop:
            self.my_tank.move(self.my_tank.direction)

    def show_enemy_tank(self):
        for e in TankGame.enemy_tank_list:
            e.random_move()
            e.tank_collide_wall()
            if e.live:
                e.display()
            else:
                TankGame.enemy_tank_list.remove(e)
                Explode(e, self.window).display_explode()
            e.random_fire()

    def show_bullet(self, ls):
        for b in ls:
            b.bullet_move()
            b.bullet_collide_wall()
            if ls is TankGame.my_bullet_list:
                b.hit_enemy_tank()
                b.bullet_collide_bullet()
            else:
                b.hit_my_tank(self.my_tank)
            if b.live:
                b.display_bullet()
            else:
                ls.remove(b)

    def get_event(self):
        global SCREEN_WIDTH, SCREEN_HEIGHT
        event_list = pygame.event.get()
        for event in event_list:
            if event.type == pygame.VIDEORESIZE:
                SCREEN_WIDTH, SCREEN_HEIGHT = event.size
                self.window = self.display.set_mode([SCREEN_WIDTH, SCREEN_HEIGHT], pygame.RESIZABLE, 32)

            if event.type == pygame.QUIT:
                self.end_game()
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_w:
                    self.my_tank.direction = U
                elif event.key == pygame.K_s:
                    self.my_tank.direction = D
                elif event.key == pygame.K_a:
                    self.my_tank.direction = L
                elif event.key == pygame.K_d:
                    self.my_tank.direction = R
                else:
                    return None
                self.my_tank.move_stop = False
            elif event.type == pygame.MOUSEBUTTONDOWN:
                if len(TankGame.my_bullet_list) < 3:
                    bullet = self.my_tank.fire()
                    load_music('fire')
                    TankGame.my_bullet_list.append(bullet)
            elif event.type == pygame.KEYUP:
                self.my_tank.move_stop = True

    def end_game(self):
        self.display.quit()
        sys.exit()

    def draw_text(self, content):
        text_sf = self.font.render(content, True, [255, 0, 0])
        return text_sf


class BaseItem(pygame.sprite.Sprite):
    def __init__(self):
        super().__init__()


class Tank(BaseItem):
    def __init__(self, left, top, window, image, direction, speed):
        super().__init__()
        self.window = window
        self.load_image = image
        self.direction = direction
        self.img = self.load_image[self.direction]
        self.rect = self.img.get_rect()
        self.rect.left = left
        self.rect.top = top
        self.speed = speed
        self.tank_width = self.rect.width
        self.tank_height = self.rect.height
        self.wall_switch = False
        self.move_stop = True
        self.live = True
        self.old_left = 0
        self.old_top = 0

    def fire(self):
        return Bullet(self, self.window)

    def display(self):
        self.img = self.load_image[self.direction]
        self.window.blit(self.img, self.rect)

    def wall_not(self, direction):
        if direction == U:
            return self.rect.top > 0
        elif direction == D:
            return self.rect.top <= SCREEN_HEIGHT - self.tank_height
        elif direction == L:
            return self.rect.left > 0
        else:
            return self.rect.left <= SCREEN_WIDTH - self.tank_width

    def wall_yes(self, direction):
        if direction == U:
            if self.rect.top < 0:
                self.rect.top = SCREEN_HEIGHT
        elif direction == D:
            self.rect.top %= SCREEN_HEIGHT
        elif direction == L:
            if self.rect.left < 0:
                self.rect.left = SCREEN_WIDTH
        else:
            self.rect.left %= SCREEN_WIDTH

    def move(self, direction):
        self.old_left = self.rect.left
        self.old_top = self.rect.top
        if self.wall_switch:
            self.wall_yes(direction)
        elif not self.wall_not(direction):
            return None
        if direction == U:
            self.rect.top -= self.speed
        elif direction == D:
            self.rect.top += self.speed
        elif direction == L:
            self.rect.left -= self.speed
        else:
            self.rect.left += self.speed

    def stay(self):
        self.rect.left = self.old_left
        self.rect.top = self.old_top

    def tank_collide_wall(self):
        for wall in TankGame.wall_list:
            if pygame.sprite.collide_rect(self, wall):
                self.stay()

    def tank_collide_tank(self):
        for tank in TankGame.enemy_tank_list:
            if pygame.sprite.collide_rect(self, tank):
                self.stay()


class MyTank(Tank):
    def __init__(self, left, top, window):
        self.img = dict(U=load_img('p2tankU'), D=load_img('p2tankD'), L=load_img('p2tankL'), R=load_img('p2tankR'))
        self.my_tank_speed = 4
        super().__init__(left, top, window, self.img, U, self.my_tank_speed)


class EnemyTank(Tank):
    def __init__(self, left, top, window):
        self.img1 = dict(U=load_img('enemy1U'), D=load_img('enemy1D'), L=load_img('enemy1L'), R=load_img('enemy1R'))
        self.img2 = dict(U=load_img('enemy2U'), D=load_img('enemy2D'), L=load_img('enemy2L'), R=load_img('enemy2R'))
        self.img3 = dict(U=load_img('enemy3U'), D=load_img('enemy3D'), L=load_img('enemy3L'), R=load_img('enemy3R'))
        self.img31 = dict(U=load_img('enemy3U_1'), D=load_img('enemy3D_1'), L=load_img('enemy3L_1'),
                          R=load_img('enemy3R_1'))
        self.img32 = dict(U=load_img('enemy3U_2'), D=load_img('enemy3D_2'), L=load_img('enemy3L_2'),
                          R=load_img('enemy3R_2'))
        # 不同的坦克击中的次数不一样
        image, self.click_count, speed = random.choice([(self.img1, 1, 4), (self.img3, 3, 3), (self.img2, 1, 5)])
        super().__init__(left, top, window, image, self.random_direction(), speed)
        self.step = 100

    @staticmethod
    def random_direction():
        n = random.randint(0, 3)
        return DIRECTION[n]

    def random_move(self):
        if self.step == 0:
            self.direction = self.random_direction()
            self.step = random.randint(10, 100)
        else:
            self.move(self.direction)
            self.step -= 1

    def random_fire(self):
        if random.randint(0, 50) == 1 and len(TankGame.enemy_bullet_list) < 30:
            enemy_bullet = self.fire()
            TankGame.enemy_bullet_list.append(enemy_bullet)


class Bullet(BaseItem):
    def __init__(self, tank, window):
        super().__init__()
        self.direction = tank.direction
        self.speed = tank.speed * 3
        self.img = load_img('bullet')
        self.rect = self.img.get_rect()
        self.window = window
        self.live = True
        if self.direction == U:
            self.rect.left = tank.rect.left + tank.rect.width / 2 - self.rect.width / 2
            self.rect.top = tank.rect.top - self.rect.height
        elif self.direction == D:
            self.rect.left = tank.rect.left + tank.rect.width / 2 - self.rect.width / 2
            self.rect.top = tank.rect.top + tank.rect.height
        elif self.direction == L:
            self.rect.left = tank.rect.left - self.rect.width / 2 - self.rect.width / 2
            self.rect.top = tank.rect.top + tank.rect.height / 2 - self.rect.height / 2
        else:
            self.rect.left = tank.rect.left + tank.rect.width
            self.rect.top = tank.rect.top + tank.rect.height / 2 - self.rect.height / 2

    def display_bullet(self):
        self.window.blit(self.img, self.rect)

    def bullet_move(self):
        if self.direction == U:
            if self.rect.top > 0:
                self.rect.top -= self.speed
                return
        elif self.direction == D:
            if self.rect.top < SCREEN_HEIGHT:
                self.rect.top += self.speed
                return
        elif self.direction == L:
            if self.rect.left > 0:
                self.rect.left -= self.speed
                return
        else:
            if self.rect.left < SCREEN_WIDTH:
                self.rect.left += self.speed
                return
        self.live = False

    def hit_enemy_tank(self):
        for enemy in TankGame.enemy_tank_list:
            hit = pygame.sprite.collide_rect(self, enemy)
            if hit:
                self.live = False
                if enemy.click_count == 1:
                    enemy.live = False
                    return None
                enemy.click_count -= 1
                if enemy.click_count == 2:
                    enemy.load_image = enemy.img32
                if enemy.click_count == 1:
                    enemy.load_image = enemy.img31
                load_music('hit')

    def hit_my_tank(self, tank):
        hit = pygame.sprite.collide_rect(self, tank)
        if hit:
            self.live = False
            tank.live = False

    def bullet_collide_wall(self):
        for wall in TankGame.wall_list:
            result = pygame.sprite.collide_rect(self, wall)
            if result:
                self.live = False
                if wall.count == 1:
                    wall.live = False
                else:
                    load_music('hit')

    def bullet_collide_bullet(self):
        for bullet in TankGame.enemy_bullet_list:
            if pygame.sprite.collide_rect(bullet, self):
                bullet.live = False
                self.live = False


class Explode(BaseItem):
    def __init__(self, tank, window):
        super().__init__()
        self.img = [load_img('blast0'), load_img('blast1'), load_img('blast2'), load_img('blast3'), load_img('blast4'),
                    load_img('blast5'), load_img('blast6')]
        self.rect = tank.rect
        self.stop = 0
        self.window = window
        self.rect.left = tank.rect.left - tank.rect.width / 2

    def display_explode(self):
        load_music('blast')
        while self.stop < len(self.img):
            self.window.blit(self.img[self.stop], self.rect)
            self.stop += 1


class Wall(BaseItem):
    def __init__(self, left, top, window):
        super().__init__()
        self.count = random.randint(0, 1)
        self.img = [load_img('steels'), load_img('walls')][self.count]
        self.rect = self.img.get_rect()
        self.rect.left = left
        self.rect.top = top
        self.window = window
        self.live = True

    def display_wall(self):
        self.window.blit(self.img, self.rect)


if __name__ == '__main__':
    g = TankGame()
    g.start_game()



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time    : 2019/7/20 13:42
# @Email  : jtyoui@qq.com
# @Software: PyCharm
from jtyoui.error import LibraryNotInstallError
from jtyoui.tools import pips
import os
import glob
import zipfile
import shutil

try:
    import fitz  # 安装 pip install PyMuPDF
except ModuleNotFoundError:
    try:
        fitz = pips('fitz', 'PyMuPDF')  # 自动安装
    except ModuleNotFoundError:
        raise LibraryNotInstallError("安装 pip install PyMuPDF")


def _get_dir_name(file_dir):
    base_name = os.path.basename(file_dir)  # 获得地址的文件名
    dir_name = os.path.dirname(file_dir)  # 获得地址的父链接
    return dir_name, base_name


def image_pdf(file_dir, pdf_address=None):
    """
    照片转pdf
    :param file_dir: 照片的地址文件夹
    :param pdf_address: 保存pdf的文件地址，默认是当前地址
    :return: 成功返回True
    """
    dir_name, base_name = _get_dir_name(file_dir)
    doc = fitz.Document()
    for img in sorted(glob.glob(file_dir + '\\*'), key=os.path.getmtime):  # 排序获得对象
        img_doc = fitz.Document(img)  # 获得图片对象
        pdf_bytes = img_doc.convertToPDF()  # 获得图片流对象
        img_pdf = fitz.Document("pdf", pdf_bytes)  # 将图片流创建单个的PDF文件
        doc.insertPDF(img_pdf)  # 将单个文件插入到文档
        img_doc.close()
        img_pdf.close()
    if not pdf_address:
        doc.save(dir_name + os.sep + base_name + ".pdf")  # 保存文档
    else:
        doc.save(pdf_address + ".pdf")  # 保存文档
    doc.close()
    return True


def pdf_image(pdf_address, image_dir=None):
    """
    PDF转照片
    :param pdf_address: PDF文件地址
    :param image_dir: 照片的文件夹地址
    :return: 成功返回True
    """
    dir_name, base_name = _get_dir_name(pdf_address)
    pdf = fitz.Document(pdf_address)
    for pg in range(0, pdf.pageCount):
        page = pdf[pg]  # 获得每一页的对象
        trans = fitz.Matrix(1.0, 1.0).preRotate(0)
        pm = page.getPixmap(matrix=trans, alpha=False)  # 获得每一页的流对象
        if not image_dir:
            pm.writePNG(str(pdf_address[:-4]) + os.sep + str(base_name[:-4]) + '_{:0>4d}.jpg'.format(pg + 1))  # 保存图片
        else:
            pm.writePNG(image_dir + os.sep + str(base_name[:-4]) + '_{:0>4d}.jpg'.format(pg + 1))  # 保存图片
    pdf.close()
    return True


def doc_to_docx(doc_path, docx_path):
    """将doc文件转为docx文件
    :param doc_path: doc文件夹的路径
    :param docx_path: 保存docx文件夹的路径
    """
    from win32com import client
    w = client.Dispatch('Word.Application')
    doc = w.Documents.Open(doc_path)
    doc.SaveAs(docx_path, 16)  # 必须有参数16，否则会出错.
    w.Quit()


def doc_to_photo(doc_path, photo_dir=None) -> bool:
    """将文件word：docx中的照片提出来,只能支持docx文档
    :param doc_path: docx文件路径
    :param photo_dir:保存照片的文件夹
    :return: 有照片返回True，没有照片返回False
    """
    dirname, name = os.path.dirname(doc_path), os.path.basename(doc_path)
    key, value = os.path.splitext(name)  # 获取名字和后缀
    if value == '.doc':
        docx_path = dirname + os.sep + key + '.docx'
        doc_to_docx(doc_path, docx_path)
        v = doc_to_photo(docx_path, photo_dir)
        os.remove(docx_path)
        return v
    if photo_dir:
        photo_path = photo_dir + os.sep + key
    else:
        photo_path = dirname + os.sep + key
    if not os.path.exists(photo_path):
        os.mkdir(photo_path)
    with zipfile.ZipFile(doc_path, 'r')as f:
        for file in f.namelist():
            if 'media' in file:  # 解压获取保存照片的地址
                f.extract(file, photo_path)
    temp = os.path.join(photo_path, 'word' + os.sep + 'media')
    if os.path.exists(temp):
        for i in os.listdir(temp):  # 移动照片地址
            shutil.copy(os.path.join(temp, i), photo_path)
        shutil.rmtree(os.path.join(photo_path, 'word'))
        return True
    print('该文件下没有照片')
    return False


image_to_pdf = image_pdf  # 照片转为PDF
pdf_to_image = pdf_image  # PDF转为照片
doc_to_image = doc_to_photo  # doc文档提取照片

if __name__ == '__main__':
    # image_pdf(r'D:\temp')  # 将照片转pdf
    # pdf_image(r'D:\temp.pdf')  # 将PDF转照片
    path = r'C:\Users\Xiaoi\Desktop\案件附件\downloadFunjian2'
    doc_to_photo(path + os.sep + '政法-刑案侦破-立案侦查.doc', path)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/6/17 17:32
# @Author: Jtyoui@qq.com
from tkinter.filedialog import *


class UI(Frame):

    def __init__(self):
        root = Tk()
        root.geometry('800x800+900+50')
        root.title('GUI')
        super().__init__(master=root)
        self.master = root
        self.pack()
        self.create_widget()
        self.master.mainloop()

    def create_widget(self):
        menubar = Menu(self)
        menu_file = Menu(menubar)
        two = Menu(menu_file)
        menubar.add_cascade(label='程序测试', menu=menu_file)
        menu_file.add_cascade(label='第二级1', menu=two)
        menu_file.add_cascade(label='第二级2', menu=two)
        two.add_command(label='第三级', command=self.test)
        self.master['menu'] = menubar

    def test(self):
        top = Toplevel()
        text = Text(top)
        text.pack()


UI()



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/9/17 10:49
# @Author: Jtyoui@qq.com
"""
由于创建Flask Docker项目文件较多，特意写了一个脚本：一键生成简单的文件信息。
"""
import os
import warnings


def create_flaskenv(project_address):
    project_name = os.path.basename(project_address)
    with open(project_address + os.sep + '.flaskenv', 'w', encoding='utf-8', newline='\n') as f:
        f.write(f'FLASK_APP = {project_name}.app\n')
        f.write('FLASK_ENV = development\n')
        f.write('FLASK_DEBUG = 0')


def create_app_sh(project_address):
    project_name = os.path.basename(project_address)
    with open(project_address + os.sep + 'app.sh', 'w', encoding='utf-8', newline='\n') as f:
        f.write('#!/bin/bash\n')
        f.write(f'uwsgi --http :5000 --ini /mnt/{project_name}/uwsgi.ini')


def create_config(project_address):
    with open(project_address + os.sep + 'config.py', 'w', encoding='utf-8', newline='\n') as f:
        import time
        t = time.strftime('%Y/%m/%d %H:%M:%S')
        text = f"""#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : {t}
# @Author: Jtyoui@qq.com
import os

BASE_DIR = os.path.dirname(os.path.abspath(__file__))  # 当前文档地址
Log = BASE_DIR + os.sep + 'logs'  # 日志存放地址


# 数据库，app配置
class Config:
    DEBUG = False
    TESTING = False
    SQLALCHEMY_TRACK_MODIFICATIONS = True
    SQLALCHEMY_POOL_RECYCLE = 10 * 60

    
class ProductionConfig(Config):
    root = ''
    password = ''
    ip = ''
    port = ''
    SQLALCHEMY_DATABASE_URI = f'mysql+pymysql://{{root}}:{{password}}@{{ip}}:{{port}}/database?charset=utf8'

  
class DevelopmentConfig(Config):
    root = ''
    password = ''
    ip = ''
    port = ''
    SQLALCHEMY_DATABASE_URI = f'mysql+pymysql://{{root}}:{{password}}@{{ip}}:{{port}}/database?charset=utf8'


class TestingConfig(Config):
    root = ''
    password = ''
    ip = ''
    port = ''
    SQLALCHEMY_DATABASE_URI = f'mysql+pymysql://{{root}}:{{password}}@{{ip}}:{{port}}/database?charset=utf8'

CONFIG = {{
    'development': DevelopmentConfig,
    'testing': TestingConfig,
    'production': ProductionConfig
}}
"""
        f.write(text)


def create_dockerfile(project_address):
    with open(project_address + os.sep + 'Dockerfile', 'w', encoding='utf-8', newline='\n') as f:
        f.write('FROM python:3.7\n')
        f.write('COPY requirements.txt /tmp/\n')
        f.write('RUN pip install --no-cache-dir -r /tmp/requirements.txt -i https://pypi.tuna.tsinghua.edu.cn/simple\n')
        f.write('RUN pip install --no-cache-dir uwsgi -i https://pypi.tuna.tsinghua.edu.cn/simple\n')
        f.write('CMD [ "python" ]\n')


def create_uwsgi(project_address):
    project_name = os.path.basename(project_address)
    with open(project_address + os.sep + 'uwsgi.ini', 'w', encoding='utf-8', newline='\n') as f:
        text = f"""[uwsgi]
project = {project_name}
base = /mnt
chdir = %(base)/%(project)
module = %(project).app:app
master = true
processes = 10
cheaper = 2
cheaper-initial = 5
cheaper-step = 1
cheaper-algo = spare
cheaper-overload = 5
"""
        f.write(text)


def create_requirements(project_address):
    with open(project_address + os.sep + 'requirements.txt', 'w', encoding='utf-8', newline='\n') as f:
        text = """Flask
PyMySQL
jtyoui
flask_sqlalchemy"""
        f.write(text)


def create_docker_init(project_address):
    import time
    t = time.strftime('%Y/%m/%d %H:%M:%S')
    project_name = os.path.basename(project_address)
    address = project_address + os.sep + project_name
    text = f"""#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : {t}
# @Author: Jtyoui@qq.com
import os
from flask import Flask
from flask_sqlalchemy import SQLAlchemy
from config import CONFIG

env = os.getenv("FLASK_ENV")
db = SQLAlchemy()
app = Flask(__name__)
app.config.from_object(CONFIG.get(env, CONFIG['development']))
db.init_app(app)
"""
    if not os.path.exists(address):
        os.mkdir(address)
    if not os.path.exists(address + os.sep + '__init__.py'):
        with open(address + os.sep + '__init__.py', 'w', encoding='utf-8')as wf:
            wf.write(text)
    else:
        warnings.warn('路径：' + address + os.sep + '__init__.py已存在。默认不覆盖，请删除后在执行！')


def create_docker_project(project_address):
    """一键创建由Flask创建的Dockers项目文件"""
    create_flaskenv(project_address)
    create_app_sh(project_address)
    create_config(project_address)
    create_dockerfile(project_address)
    create_requirements(project_address)
    create_uwsgi(project_address)
    create_docker_init(project_address)


if __name__ == '__main__':
    create_docker_project(os.path.dirname(__file__))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/4/22 11:41
# @Author: Jtyoui@qq.com
import zipfile
from os import path

file_zip_path = path.dirname(path.abspath(__file__))
sep = path.sep


def load_zip(zip_name, file_name, encoding='UTF-8', sep='\n'):
    """加载zip数据
    :param zip_name:压缩包的名字
    :param file_name:压缩包里面文件的名字
    :param encoding:文件的编码
    :param sep:压缩文件里面的换行符
    :return: 压缩包里面的数据：默认编码的UTF-8
    """
    file_zip = path.join(file_zip_path, zip_name)
    f = zipfile.ZipFile(file_zip)
    fp = f.read(file_name)
    lines = fp.decode(encoding).split(sep)
    return lines


if __name__ == '__main__':
    line = load_zip('train.zip', 'train.txt')
    for l in line:
        print(l)

# 关于文件压缩包目录
# py.zip是汉语拼音
# train.zip是中国火车站名字压缩包
# city.zip是中国天气预报城市与编码



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/8/1 14:49
# @Author: Jtyoui@qq.com
from keras.models import Model
from keras.layers import Embedding, Bidirectional, LSTM, TimeDistributed
from keras.layers import Input, Dropout, Conv1D, Dense, concatenate
from keras_contrib.layers.crf import CRF
from jtyoui.neuralNetwork.kerase.AnalyticalData import *
from keras.utils.np_utils import np
from keras.optimizers import Adam

chunk_tags = {'O': 0, 'B-a': 1, 'I-a': 2, 'B-b': 3, 'I-b': 4, 'B-c': 5, 'I-c': 6}
dirs = r'C:\Users\Xiaoi\Desktop\datagrand\datagrand'
vocab_path_model = dirs + '/vocab.pkl'
train_vocab_path = dirs + '/corpus.txt'
train_path = dirs + '/train.txt'
test_path = dirs + '/test.txt'
temp_path = dirs + '/crf.txt'
result_path = dirs + '/result.txt'
model_path = dirs + '/model-ner.h5'
vocab = load_vocab(vocab_path_model)
length = analysis_rational_len(train_path, percent=0.93)

EMBED_DIM = 300
HALF_WIN_SIZE = 2
DROPOUT_RATE = 0.1
FILTERS = 50
DENSE_DIM = 50


def create_model():
    inputs = Input(shape=(length,), dtype='int32', name='inputs')
    embedding_1 = Embedding(len(vocab), EMBED_DIM, input_length=length, mask_zero=True)(inputs)
    bilstm = Bidirectional(LSTM(EMBED_DIM // 2, return_sequences=True))(embedding_1)
    bilstm_dropout = Dropout(DROPOUT_RATE)(bilstm)
    embedding_2 = Embedding(len(vocab), EMBED_DIM, input_length=length)(inputs)
    con = Conv1D(filters=FILTERS, kernel_size=2 * HALF_WIN_SIZE + 1, padding='same')(embedding_2)
    con_d = Dropout(DROPOUT_RATE)(con)
    dense_con = TimeDistributed(Dense(DENSE_DIM))(con_d)
    rnn_cnn = concatenate([bilstm_dropout, dense_con], axis=2)
    dense = TimeDistributed(Dense(len(chunk_tags)))(rnn_cnn)
    crf = CRF(len(chunk_tags), sparse_target=True)
    crf_output = crf(dense)
    model = Model(input=[inputs], output=[crf_output])
    model.compile(loss=crf.loss_function, optimizer=Adam(), metrics=[crf.accuracy])
    return model


def train_model(model):
    train, label = vocab_train_label(train_path, vocab=vocab, tags=chunk_tags, max_chunk_length=length)
    n = np.array(label, dtype=np.float)
    label = n.reshape((n.shape[0], n.shape[1], 1))
    train = np.array(train)
    model.fit(x=train, y=label, batch_size=16, epochs=4)
    model.save(model_path)
    return True


def predict_model(model):
    x_test, original = vocab_test(test_path, vocab, length)
    ws = open(temp_path, mode='w', newline='\n')
    tags = dict(zip(chunk_tags.values(), chunk_tags.keys()))
    model.load_weights(model_path)
    for question, tests in zip(original, x_test):
        raw = model.predict([[tests]])[0][-len(question):]
        result = [np.argmax(row) for row in raw]
        answer = tuple(map(lambda x: tags[x], result))
        ma = map(lambda x: x[0] + '\t' + x[1] + '\n', zip(question, answer))
        ws.writelines(ma)
        ws.write('\n')
    ws.flush()
    ws.close()


if __name__ == '__main__':
    # m = create_model()
    # train_model(m)
    # predict_model(m)
    restore_format(crf_path=temp_path, standard_path=result_path)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/11/25 14:19
# @Author: Jtyoui@qq.com
import jtyoui.neuralNetwork
import jtyoui
import re
import os


class EntityExtraction:
    def __init__(self, sentence: str, model_path: str = None):
        self.sentence = sentence
        self.num = None
        self.model_path = model_path
        if self.model_path and os.path.exists(self.model_path):
            self._st = jtyoui.neuralNetwork.ernie_st(self.model_path)
            self._load()

    def _nn(self, r):
        """根据正则去获取实体"""
        ls = self.num
        word = []
        if ls:
            ls = jtyoui.join('', ls)
            for index in re.finditer(r, ls):
                word.append(self.sentence[index.start():index.end()])
        return word

    @property
    def people(self):
        """提取人名"""
        return self._nn('[01]+')

    @property
    def address(self):
        """提取地址"""
        return self._nn('[45]+')

    @property
    def org(self):
        """提取机构名"""
        return self._nn('[23]+')

    def _load(self):
        """加载模型"""
        self.num = jtyoui.neuralNetwork.ernie_match(self.sentence, self._st)

    @property
    def time(self):
        """提取时间"""
        p = jtyoui.ParseTime(self.sentence)
        return p.find_times()

    @property
    def re_num(self):
        """提取数字"""
        return re.findall(r'\d+', self.sentence)

    @property
    def car_plate(self):
        """提取车牌号码"""
        return jtyoui.plate_number(self.sentence)

    @property
    def phone(self):
        """提取手机号码"""
        t = jtyoui.telephone_number_matching_verification(jtyoui.ALL_Mobile_Data_Network_Card_RE, self.sentence)
        return [i for i in t]

    @property
    def re_card(self):
        """提取身份证号码"""
        cards = []
        for i in re.findall(r'\d{18}|\d{17}[Xx]|\d{15}', self.sentence):
            try:
                jtyoui.check_id_card(i)
                cards.append(i)
            except jtyoui.IdCardCheckError:
                ...
            except AssertionError:
                ...
        return cards

    def set_sentence(self, sentence):
        """从定义语句"""
        self.sentence = sentence
        if self.model_path:
            self._load()

    @property
    def sentences(self):
        return self.sentence

    @sentences.setter
    def sentences(self, sentence):
        self.set_sentence(sentence)


if __name__ == '__main__':
    ee = EntityExtraction(
        '李斯从金阳世纪城打到中天铭廷，他的车牌是：贵AU8080。并且他的电话是：15180864970，身份证号码是：522121193702157024，时间是昨天下午2点半，他在花溪公园玩耍',
        model_path='D://model')
    print(ee.time)
    print(ee.address)
    print(ee.car_plate)
    print(ee.org)
    print(ee.people)
    print(ee.phone)
    print(ee.re_card)
    print(ee.re_num)
    ee.sentence = '今天'
    print(ee.time)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time    : 2019/5/24 23:10
# @Email  : jtyoui@qq.com
# @Software: PyCharm
from abc import ABC
from html.parser import HTMLParser


class ParseHtml(HTMLParser, ABC):
    """基类"""

    def __init__(self, start_tag, start_attr, end_tag, end_attr):
        """使用相当简单.
            html = '<div class="declare" id="J-declare">声明：百科词条人人可编辑。<a class="declare-details"></a>'
            p = ParseHtml('div', ['class="declare"'], 'a', ['class="declare-details"'])
            p.feed(html)
            print(p.get_data())  # 声明：百科词条人人可编辑。
        :param start_tag:开始标签，必须填写
        :param start_attr:开始标签里面的属性，切记一定是列表[]类型。如果没有，传入空列表[]
        :param end_tag:结束标签，必须填写
        :param end_attr:结束标签里面的属性，切记一定是列表[]类型。如果没有，传入空列表[]
        """
        HTMLParser.__init__(self)
        self._data = ''
        self._flag = False
        self._start_tag = start_tag
        self._start_attr = self._split(start_attr)
        self._end_tag = end_tag
        self._end_attr = self._split(end_attr)

    def handle_starttag(self, tag, attrs):  # 开始标签
        for attr in self._start_attr:
            if attr not in attrs:
                break
        else:
            if tag == self._start_tag:
                self._flag = True
                return

        for att in self._end_attr:
            if att not in attrs:
                break
        else:
            if tag == self._end_tag:
                self._flag = False
                return

    def handle_endtag(self, tag):
        if not self._end_attr:
            if tag == self._end_tag:
                self._flag = False

    def handle_data(self, data):  # 内容
        if self._flag:
            self._data += data

    def get_data(self):
        """得到数据"""
        return self._data.strip()

    @staticmethod
    def _split(words):
        total = []
        for word in words:
            tag, attr = word.split('=', 1)
            total.append((tag, attr.replace('"', '')))
        return total


if __name__ == '__main__':
    html = '<div class="declare" id="J-declare">声明：百科词条人人可编辑。<a class="declare-details"></a>'
    p = ParseHtml('div', ['class="declare"'], 'a', ['class="declare-details"'])
    p.feed(html)
    print(p.get_data())  # 声明：百科词条人人可编辑。



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/6/11 16:06
# @Author: Jtyoui@qq.com
from jtyoui.web import headers_ua
from jtyoui.error import LibraryNotInstallError
import requests

"""爬虫网站。请求Get和Post封装"""


def get(url, cookie=None):
    """Get网站"""
    headers = headers_ua()
    if cookie:
        headers['cookie'] = cookie
    response = requests.get(url=url, headers=headers)
    return response


def post(url, params, cookie=None):
    """Post网站"""
    headers = headers_ua()
    if cookie:
        headers['cookie'] = cookie
    response = requests.post(url=url, data=params, headers=headers)
    return response


def get_js(js, js_fun, js_params):
    try:
        import execjs  # pip install PyExecJS
    except:
        raise LibraryNotInstallError('安装：pip install PyExecJS')
    ctx = execjs.compile(js)  # 加载JS文件
    return ctx.call(js_fun, *js_params)  # 调用js方法  第一个参数是JS的方法名，后面的data和key是js方法的参数


if __name__ == '__main__':
    print(get('http://www.27k.cc/').text)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/18 9:36
# @Author: Jtyoui@qq.com
from jtyoui.decorator import replace_regular
from jtyoui.regular import punctuation_re

import hashlib

"""
海明距离
在信息编码中，两个合法代码对应位上编码不同的位数称为码距，又称海明距离。
举例如下：10101和00110从第一位开始依次有第一位、第四、第五位不同，则海明距离为3。
"""


def handle(participle_ls, weight, f):
    """
    将内容转成字典格式
    :param participle_ls: 文本分词内容,是一个list分词对象
    :param weight: 特征值
    :param f: simHash的bit位数
    :return: 海明距离值
    """
    c = []
    for ls in participle_ls:
        if weight:
            v = weight.get(ls, 1)
        else:
            v = 1
        c.append((ls, v))
    return features_dict(c, f)


def hash_func(x):
    """hash算法"""
    return int(hashlib.md5(x).hexdigest(), 16)


def features_dict(features, f):
    """
    特征值字典
    :param features: 特征值
    :param f: simHash的bit位数
    :return: simHash值
    """
    v = [0] * f
    masks = [1 << i for i in range(f)]
    for values, weight in features:
        h = hash_func(values.encode('utf-8'))
        for i in range(f):
            v[i] += weight if h & masks[i] else -weight
    value = 0
    for i in range(f):
        if v[i] > 0:
            value |= masks[i]
    return value


def distance(sim_hash, another, f):
    """
    计算两个simHash的距离
    :param sim_hash: simHash值
    :param another: 另一个simHash的值
    :param f: simHash的bit位数
    :return: 海明距离
    """
    x = (sim_hash ^ another) & ((1 << f) - 1)
    value = 0
    while x:
        value += 1
        x &= x - 1
    return value


def ham_distance(chars, other_chars, weight=None, f=64):
    """
    比较那个字符串的海明距离
    :param chars: 字符串
    :param other_chars: 另一个字符串
    :param weight: 权重字典:weight={"电影": 3}
    :param f: samHash的bit位数
    :return: 海明距离值
    """
    v0 = handle(chars, weight, f)
    v1 = handle(other_chars, weight, f)
    return distance(v0, v1, f)


@replace_regular(punctuation_re, '')
def simHash_similarity(text1: (str, dict), text2: (str, dict), weight: dict = None, f: int = 64) -> float:
    """文本相似度算法
    :param text1: 文本1
    :param text2: 文本2
    :param weight: 文本词权重
    :param f: hash bit位数
    :return: 相似度
    """
    v = ham_distance(text1, text2, weight=weight, f=f)
    return 1 - v / f


if __name__ == '__main__':
    a = ['我', '吃饭', '了', '明天', '去', '看', '电影']
    b = ['我', '在', '吃饭', '了', '马上', '去', '看', '电影']
    print(ham_distance(a, b, weight={"电影": 3}))
    print(simHash_similarity(''.join(a), ''.join(b)))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/14 9:06
# @Author: Jtyoui
from jtyoui.web import random
from jtyoui.error import BaiDuMapError
import requests
from random import choice
from math import ceil

_total_data = []
AK = ['8R82NkA5j2YzCO1hG2grXUxdLQnnHdVA', 'mSYhaXLLGXffSkkgkalK84RV0Aof22vA', 'OpYA7uSs7czqHw68w07ZzE9t08RQWrfO',
      'Tffez86k4u6rwGmE3MPR2N3XtUUNbZ7H', 'rINxBQe6h4OGbmrLiImffAj96ZlhxCY8', 'V6QYkOxaGwvU3WDVZKBdNUVjVvLcwQpH']


class Data:
    def __init__(self, name, location, address):
        """
        爬取的数据类
        :param name: 名称
        :param location: 百度坐标系
        :param address: 名称的详细地址
        """
        self.name = name
        self.location = location
        self.address = address

    def __repr__(self):
        return '名字:' + self.name + '\t坐标:' + self.location + '\t地址:' + self.address

    def __str__(self):
        return '名字:' + self.name + '\t坐标:' + self.location + '\t地址:' + self.address


def get_data(title, scope, page_size=20, page_num=0):
    """
    获取数据
    :param title: 主题
    :param scope: 范围
    :param page_size: 返回一条是数据,默认是20,最大是20
    :param page_num: 页数
    :return: 主题信息集合
    """
    global _total_data
    ak = choice(AK)  # 随机选取一个Ak值.AK要在百度接口上获取
    address = f'http://api.map.baidu.com/place/v2/search?query={title}&region={scope}&output=json&ak={ak}&page_size=' \
        f'{page_size}&page_num={page_num}'
    r = requests.get(address, headers={'User-Agent': random()})
    json = r.json()
    status = json.get('status')
    if status == 401:
        get_data(title, scope, page_num=page_num)
        return None
    elif str(status).startswith('3'):
        raise BaiDuMapError('该模块已经废弃不可用')
    print(json)
    results = json['results']  # 获取主要信息
    total = json['total']  # 获取总条数
    page = ceil(total / page_size)  # 取上整,获取页数
    current_page = page_num + 1  # 当前页数,应该等于上一页加1
    for result in results:
        name = result['name']
        locations = result['location']
        location = str(locations['lng']) + '|' + str(locations['lat'])
        address = result.get('province') + result.get('city') + result.get('area') + result.get('address')
        _total_data.append(Data(name, location, address))
    if current_page < page:
        get_data(title, scope, page_num=current_page)


def bd_map(title, scope, page_size=20, page_num=0):
    """获取信息,和get_data()一样"""
    get_data(title, scope, page_size, page_num)
    return _total_data


def save_txt(file):
    """
    保存到纯文本
    :param file: 文件地址
    """
    with open(file, encoding='utf-8', mode='a') as f:
        for d in _total_data:
            f.write(str(d) + '\n')


def save_csv(file):
    """
    保存到csv文件
    :param file: csv文件地址
    """
    with open(file, encoding='GBK', mode='w') as f:
        for d in _total_data:
            d = str(d).replace('\t', ',')
            f.write(d + '\n')


if __name__ == '__main__':
    bd = bd_map('景区', '黔东南苗族侗族自治州')
    # save_txt('./景区.txt')
    save_csv('./景区.csv')



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/9/25 15:12
# @Author: Jtyoui@qq.com
from jtyoui.error import CoordinateLengthNotEqualError


def map_replace(str_: str, key: [list, str] = None, value: [list, str] = None, maps: dict = None) -> str:
    """映射替换,最好使用maps字典映射
    :param str_: 字符串
    :param key: 替换字符
    :param value: 被替换的字符
    :param maps:字符映射
    :return: 替换完毕的字符串
    """
    if maps:
        strings = str_
        for k, v in maps.items():
            strings = strings.replace(k, v)
        return strings
    else:
        if len(key) != len(value):
            raise CoordinateLengthNotEqualError('替换的长度必须相同！')
        if isinstance(key, list):
            key = ''.join(key)
        if isinstance(value, list):
            value = ''.join(value)
    m = str.maketrans(key, value)
    return str_.translate(m)


if __name__ == '__main__':
    print(map_replace('[中国]', '[]', '【】'))
    print(map_replace('[中国]', maps={'[': '【', ']': '】'}))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/18 9:34
# @Author: Jtyoui@qq.com
from jtyoui import parameter_set_length

"""
切比雪夫距离(棋盘距离)
在数学中，切比雪夫距离或是L∞度量，是向量空间中的一种度量，
二个点之间的距离定义是其各坐标数值差绝对值的最大值。以数学的观点来看，切比雪夫距离是由一致范数（uniform norm）
所衍生的度量，也是超凸度量的一种。
国际象棋中，国王可以直行、横行、斜行，所以国王走一步可以移动到相邻8个方格中的任意一个。
国王从格子(x1,y1)走到格子(x2,y2)最少需要多少步？这个距离就叫切比雪夫距离。
"""


@parameter_set_length
def chebyshev_distance(coordinate_p, coordinate_q):
    """
    传入的是两个可迭代对象,每一个是一个n位坐标,比如:p=(x1,x2,x3....xn),q=(y1,y2,y3....yn)
    :param coordinate_p: p坐标
    :param coordinate_q: q坐标
    :return: 切比雪夫距离(棋盘距离)
    """
    coordinate = []
    for x, y in zip(coordinate_p, coordinate_q):
        coordinate.append(abs(x - y))
    return max(coordinate)


if __name__ == '__main__':
    c_d = chebyshev_distance((11, 2, 2, 3, 23, 2), (3, 3, 2, 3, 2, 3))
    print(c_d)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time : 2019/3/23 0023
# @Email : jtyoui@qq.com

"""处理二维矩阵"""
from copy import deepcopy
from jtyoui.error import MatrixNotDottedError


class Matrix:

    def __init__(self, data):
        if not isinstance(data, list):
            raise TypeError("传入list类型")
        self.__data = data
        self.__length = len(data)

    @staticmethod
    def _checking(other) -> list:
        if isinstance(other, Matrix):
            other = deepcopy(other.__data)
        elif not isinstance(other, list):
            raise TypeError("传入list类型")
        return other

    def __mul__(self, other):
        dot = []
        other = self._checking(other)
        other_w, other_h = len(other[0]), len(other)
        data_w, data_h = len(self.__data[0]), self.__length
        if data_w != other_h or data_h != other_w:
            raise MatrixNotDottedError("矩阵不能点乘")

        for data_j in range(data_h):
            dot.append([])
            for i in range(other_w):
                all_ = [self.__data[data_j][j] * other[j][i] for j in range(other_h)]
                dot[data_j].append(sum(all_))
        return dot

    def __add__(self, other):
        dot = []
        other = self._checking(other)
        other_w, other_h = len(other[0]), len(other)
        data_w, data_h = len(self.__data[0]), self.__length
        if data_w != other_w or other_h != data_h:
            raise MatrixNotDottedError("矩阵不能相加")
        for j in range(data_h):
            row = []
            for i in range(other_w):
                all_ = self.__data[j][i] + other[j][i]
                row.append(all_)
            dot.append(row)
        return dot

    def __iadd__(self, other):
        return self.__add__(other)

    def __sub__(self, other):
        other = self._checking(other)
        length = len(other)
        for i in range(length):
            for j in range(length):
                other[i][j] = -other[i][j]
        return self.__add__(other)

    def __isub__(self, other):
        return self.__sub__(other)

    @property
    def i(self):
        """逆矩阵"""
        n = self.__length
        original = deepcopy(self.__data)
        i_ = self.unitized(n)  # 获得单位矩阵
        swap, triangle = self.elimination(original)  # 消元
        diagonal = self.diagonal_unitized(original)  # 对角单位化
        trace = self.trace_vector(original)

        for i in range(n):
            if swap[i] != i:
                i_[i], i_[swap[i]] = i_[swap[i]], i_[i]
            for j in range(i + 1, n):
                for k in range(n):
                    if triangle[j][i]:
                        i_[j][k] = i_[j][k] - triangle[j][i] * i_[i][k]
        for i in range(n - 1):
            for j in range(n - 1 - i):
                if diagonal[n - 1 - i - j - 1][n - 1 - i]:
                    for k in range(n):
                        i_[n - 1 - i - j - 1][k] -= diagonal[n - 1 - i - j - 1][n - 1 - i] * i_[n - 1 - i][k]
        for i in range(n):
            for j in range(n):
                i_[i][j] = i_[i][j] / trace[i]
        return i_

    @property
    def t(self):
        """转置矩阵"""
        transposition = []
        for i in range(len(self.__data[0])):
            transposition.append([self.__data[j][i] for j in range(self.__length)])
        return transposition

    @property
    def det(self):
        """秩"""
        return 1

    def unitized(self, length):
        """单位E矩阵"""
        eye = self.zero_matrix(length)
        for i in range(length):
            eye[i][i] = 1
        return eye

    def elimination(self, data):
        """消元矩阵"""
        """交换操作记录数组"""
        swap, triangle = list(range(self.__length)), self.zero_matrix(self.__length)  # 交换矩阵 三角矩阵

        """对每一列进行操作"""
        for i in range(self.__length):
            max_row, row = data[i][i], i
            for j in range(i, self.__length):
                if data[j][i] >= max_row:
                    max_row, row = data[j][i], j
            swap[i] = row

            """交换"""
            if row != i:
                for j in range(self.__length):
                    data[i][j], data[row][j] = data[row][j], data[i][j]

            """消元"""
            for j in range(i + 1, self.__length):
                if data[j][i]:
                    triangle[j][i] = data[j][i] / data[i][i]
                    for k in range(self.__length):
                        data[j][k] = data[j][k] - triangle[j][i] * data[i][k]

        return swap, triangle

    def diagonal_unitized(self, data):
        """对角单位化"""
        diagonal, position = self.zero_matrix(self.__length), self.__length - 1  # 对角矩阵

        for i in range(position):
            for j in range(position - i):
                if data[position - i - j - 1][position - i] and data[position - i][position - i]:
                    diagonal[position - i - j - 1][position - i] = (
                            data[position - i - j - 1][position - i] / data[position - i][position - i])
                    for k in range(self.__length):
                        data[position - i - j - 1][k] = data[position - i - j - 1][k] - (
                                diagonal[position - i - j - 1][position - i] * data[position - i][k])

        return diagonal

    def trace_vector(self, data):
        """矩阵迹向量"""
        return [data[i][i] for i in range(self.__length)]

    @staticmethod
    def zero_matrix(length):
        """单位零矩阵"""
        eye = []
        for i in range(length):
            eye.append([0 for _ in range(length)])
        return eye

    def __len__(self):
        return len(self.__data)

    def __getitem__(self, item):
        pass


if __name__ == '__main__':
    d = [
        [1, 2, 3],
        [1, 0, -1],
        [0, 1, 1]
    ]
    m1 = Matrix(d)
    m2 = Matrix(d)
    print(m1 * m2)
    print(m1 + m2)
    print(m1 - m2)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time    : 2019/4/28 18:35
# @Email  : jtyoui@qq.com
# @Software: PyCharm
from jtyoui.baidu import BaiDuInfoSearch
from jtyoui.error import NameOfTrainStationNotFoundError
from jtyoui.file_zip import load_zip
from jtyoui.baidu import Load_BaiDuBaiKe

_lines = load_zip('train.zip', 'train.txt')

_train, _province = [], []
_flag = False
for _line in _lines:
    if '----' not in _line:
        if _flag:
            _province.append(_line)
        else:
            _train.append(_line)
    else:
        _flag = True

Train_Station = dict(zip(_train, _province))


def _find_train_station_(name):
    if not ('火车站' in name or '站' in name):
        if Train_Station.get(name):
            name += '站'
        else:
            raise NameOfTrainStationNotFoundError(F"没有找到{name}火车站信息，请正确输入！")
    text = Load_BaiDuBaiKe(name)
    if '站' in text:
        return text
    return ''


def find_train_info(name):
    """查询火车站的基本信息"""
    data = _find_train_station_(name)
    bd = BaiDuInfoSearch(data)
    return bd.info()


def find_train_desc(name):
    """查询火车站的摘要信息"""
    data = _find_train_station_(name)
    bd = BaiDuInfoSearch(data)
    return bd.desc()


def find_train_desc_info(name):
    """查询火车站的摘要和基本信息"""
    data = _find_train_station_(name)
    bd = BaiDuInfoSearch(data)
    return bd.desc(), bd.info()


if __name__ == '__main__':
    print(Train_Station['安顺'])  # 查看安顺火车站是哪个省
    desc = find_train_desc('安顺火车站')  # 查看安顺火车站的摘要
    print(desc)
    info = find_train_info('安顺站')  # 查询安顺火车站的基本信息
    print(info)
    di = desc_info = find_train_desc_info('宋')
    print(di)



#!///3.7
# -*- : -8 -*-
# @  : 2019/10/11 15:05
# @: @.

SurName = """
赵 钱 孙 李 周 吴 郑 王 冯 陈 褚 卫
蒋 沈 韩 杨 朱 秦 尤 许 何 吕 施 张
孔 曹 严 华 金 魏 陶 姜 戚 谢 邹 喻
柏 水 窦 章 云 苏 潘 葛 奚 范 彭 郎
鲁 韦 昌 马 苗 凤 花 方 俞 任 袁 柳
酆 鲍 史 唐 费 廉 岑 薛 雷 贺 倪 汤
滕 殷 罗 毕 郝 邬 安 常 乐 于 时 傅
皮 卞 齐 康 伍 余 元 卜 顾 孟 平 黄
和 穆 萧 尹 姚 邵 湛 汪 祁 毛 禹 狄
米 贝 明 臧 计 伏 成 戴 谈 宋 茅 庞
熊 纪 舒 屈 项 祝 董 梁 杜 阮 蓝 闵
席 季 麻 强 贾 路 娄 危 江 童 颜 郭
梅 盛 林 刁 锺 徐 邱 骆 高 夏 蔡 田
樊 胡 凌 霍 虞 万 支 柯 昝 管 卢 莫
经 房 裘 缪 干 解 应 宗 丁 宣 贲 邓
郁 单 杭 洪 包 诸 左 石 崔 吉 钮 龚
程 嵇 邢 滑 裴 陆 荣 翁 荀 羊 於 惠
甄 麴 家 封 芮 羿 储 靳 汲 邴 糜 松
井 段 富 巫 乌 焦 巴 弓 牧 隗 山 谷
车 侯 宓 蓬 全 郗 班 仰 秋 仲 伊 宫
宁 仇 栾 暴 甘 钭 历 戎 祖 武 符 刘
景 詹 束 龙 叶 幸 司 韶 郜 黎 蓟 溥
印 宿 白 怀 蒲 邰 从 鄂 索 咸 籍 赖
卓 蔺 屠 蒙 池 乔 阳 郁 胥 能 苍 双
闻 莘 党 翟 谭 贡 劳 逄 姬 申 扶 堵
冉 宰 郦 雍 却 璩 桑 桂 濮 牛 寿 通
边 扈 燕 冀 僪 浦 尚 农 温 别 庄 晏
柴 瞿 阎 充 慕 连 茹 习 宦 艾 鱼 容
向 古 易 慎 戈 廖 庾 终 暨 居 衡 步
都 耿 满 弘 匡 国 文 寇 广 禄 阙 东
欧 殳 沃 利 蔚 越 夔 隆 师 巩 厍 聂
晁 勾 敖 融 冷 訾 辛 阚 那 简 饶 空
曾 毋 沙 乜 养 鞠 须 丰 巢 关 蒯 相
查 后 荆 红 游 竺 权 逮 盍 益 桓 公
万俟 司马 上官 欧阳 夏侯 诸葛 闻人 东方 赫连 皇甫 尉迟 公羊
澹台 公冶 宗政 濮阳 淳于 单于 太叔 申屠 公孙 仲孙 轩辕 令狐
钟离 宇文 长孙 慕容 司徒 司空 召 有 舜 叶赫那拉 丛 岳
寸 贰 皇 侨 彤 竭 端 赫 实 甫 集 象
翠 狂 辟 典 良 函 芒 苦 其 京 中 夕
之 章佳 那拉 冠 宾 香 果 依尔根觉罗 依尔觉罗 萨嘛喇 赫舍里 额尔德特
萨克达 钮祜禄 他塔喇 喜塔腊 讷殷富察 叶赫那兰 库雅喇 瓜尔佳 舒穆禄 爱新觉罗 索绰络 纳喇
乌雅 范姜 碧鲁 张廖 张简 图门 太史 公叔 乌孙 完颜 马佳 佟佳
富察 费莫 蹇 称 诺 来 多 繁 戊 朴 回 毓
税 荤 靖 绪 愈 硕 牢 买 但 巧 枚 撒
泰 秘 亥 绍 以 壬 森 斋 释 奕 姒 朋
求 羽 用 占 真 穰 翦 闾 漆 贵 代 贯
旁 崇 栋 告 休 褒 谏 锐 皋 闳 在 歧
禾 示 是 委 钊 频 嬴 呼 大 威 昂 律
冒 保 系 抄 定 化 莱 校 么 抗 祢 綦
悟 宏 功 庚 务 敏 捷 拱 兆 丑 丙 畅
苟 随 类 卯 俟 友 答 乙 允 甲 留 尾
佼 玄 乘 裔 延 植 环 矫 赛 昔 侍 度
旷 遇 偶 前 由 咎 塞 敛 受 泷 袭 衅
叔 圣 御 夫 仆 镇 藩 邸 府 掌 首 员
焉 戏 可 智 尔 凭 悉 进 笃 厚 仁 业
肇 资 合 仍 九 衷 哀 刑 俎 仵 圭 夷
徭 蛮 汗 孛 乾 帖 罕 洛 淦 洋 邶 郸
郯 邗 邛 剑 虢 隋 蒿 茆 菅 苌 树 桐
锁 钟 机 盘 铎 斛 玉 线 针 箕 庹 绳
磨 蒉 瓮 弭 刀 疏 牵 浑 恽 势 世 仝
同 蚁 止 戢 睢 冼 种 涂 肖 己 泣 潜
卷 脱 谬 蹉 赧 浮 顿 说 次 错 念 夙
斯 完 丹 表 聊 源 姓 吾 寻 展 出 不
户 闭 才 无 书 学 愚 本 性 雪 霜 烟
寒 少 字 桥 板 斐 独 千 诗 嘉 扬 善
揭 祈 析 赤 紫 青 柔 刚 奇 拜 佛 陀
弥 阿 素 长 僧 隐 仙 隽 宇 祭 酒 淡
塔 琦 闪 始 星 南 天 接 波 碧 速 禚
腾 潮 镜 似 澄 潭 謇 纵 渠 奈 风 春
濯 沐 茂 英 兰 檀 藤 枝 检 生 折 登
驹 骑 貊 虎 肥 鹿 雀 野 禽 飞 节 宜
鲜 粟 栗 豆 帛 官 布 衣 藏 宝 钞 银
门 盈 庆 喜 及 普 建 营 巨 望 希 道
载 声 漫 犁 力 贸 勤 革 改 兴 亓 睦
修 信 闽 北 守 坚 勇 汉 练 尉 士 旅
五 令 将 旗 军 行 奉 敬 恭 仪 母 堂
丘 义 礼 慈 孝 理 伦 卿 问 永 辉 位
让 尧 依 犹 介 承 市 所 苑 杞 剧 第
零 谌 招 续 达 忻 六 鄞 战 迟 候 宛
励 粘 萨 邝 覃 辜 初 楼 城 区 局 台
原 考 妫 纳 泉 老 清 德 卑 过 麦 曲
竹 百 福 言 第五 佟 爱 年 笪 谯 哈 墨
南宫 赏 伯 佴 佘 牟 商 西门 东门 左丘 梁丘 琴
后 况 亢 缑 帅 微生 羊舌 海 归 呼延 南门 东郭
百里 钦 鄢 汝 法 闫 楚 晋 谷梁 宰父 夹谷 拓跋
壤驷 乐正 漆雕 公西 巫马 端木 颛孙 子车 督 仉 司寇 亓官
鲜于 锺离 盖 逯 库 郏 逢 阴 薄 厉 稽 闾丘
公良 段干 开 光 操 瑞 眭 泥 运 摩 伟 铁 迮
荔菲 辗迟
""".strip().split()

_message = """
    赵：造父为周穆王驾车，穆王把赵城赐了给他，其后代以国为姓。赵匡胤是宋代的开国皇帝，据说赵姓放在第一位与此有关。
    钱：彭祖的孙子彭孚在西周朝廷中任钱府上士，其后人以其官为姓。
    孙：周文王的一后代叫惠孙，他的后代以他的名字“孙”作为姓。
    李：皋陶的后人理征因得罪纣王被处死，其妻儿在一棵李子树下摘果充饥得以活命，其后人为纪念李子的救命之恩改为“李”姓。
    周：相传黄帝时就有一位叫周昌的大将，至商代又有一名叫周任的太史，这两个人的后代都以周为姓氏，还有以周文王或者周公的部分后人也以周为姓，以及唐朝时为避唐玄宗名讳，姬姓改为周姓。
    吴：周太王亶父之子太伯、仲雍建立吴国，其子孙称为吴氏。
    郑：郑桓公遗族以国名为姓。
    王：出自姬姓，为周文王第十五子毕公高的后代。
    冯：毕公高的后人就以封地为姓。
    陈：最早出自妫姓，胡公满的子孙们以国为姓。
    褚：恭段受封于褚邑，子孙就都以“褚”为姓。
    卫：周文王的第九个儿子叫康叔，建立了卫国，他的后代以国名为姓。
    蒋：周武王的兄弟伯龄，被封到“蒋”，公族子孙以国名为姓。
    沈：周文王的第十个儿子叫季载，成立沈国。沈国君主的后代都以沈为姓。
    韩：周时贵族武子，受封于韩原，子姓以地名为姓：战国时的韩国，被秦国并，原韩国王族世代姓韩。
    杨：上古时候，周宣王的儿子尚父，封到杨邑，称做杨侯，建杨国，子孙用杨姓。
    朱：西周时，周武王封曹侠到邾，后代以地名为姓，写作朱。
    秦：上古的非子善于驯养马匹，被周孝王封为秦地的首领，建立了秦国。其后人以国为姓。
    尤：尤姓是由沈姓而来。
    许：远古炎帝的后代。文叔建立了许国，后代就以许为姓。
    何：春秋战国时，韩国的公族中有一支迁移至江淮，以国为姓。
    吕：炎帝后代伯夷被封为吕侯，他的后代就用吕作为姓。
    施：夏朝时，有诸侯国，称施国，今湖北恩施县，其后代姓施。
    张：春秋时晋国贵族叫解张，他的后代用他的名作为姓。
    孔：开国君主叫天乙，后将乙字和子字并拢，形成孔字，定为姓。
    曹：远古有一贵族叫“安”，帮助大禹治水，被恩赐给一个曹姓。
    严：严姓是由庄姓变来。
    华：西周时候，宋国公子，受封于“华”，他的子孙后来就以地名为姓。
    金：远古黄帝的儿子少昊的后代以金为姓。
    魏：毕万的后代毕斯建起魏国，称魏文侯，其后人以国为姓。
    陶：尧的子孙中，有人以制陶为荣耀，就姓陶，世代延续。
    姜：以地名为姓。
    戚：以地名为姓。
    谢：用地名做姓。
    邹：最早出自姚姓，以地名为姓。姓名名典《姓觽》：邹国，舜后，姚姓。舜帝后裔姚姓族人开创建有邹国，是最早的邹国。
    喻：远古黄帝时，有个医官叫俞柎，为喻姓祖先。
    柏：柏姓起源非常古老，黄帝有个孙子叫颛顼，他的老师就姓柏。
    水：水姓是大禹的后代，大禹治水，他的族人都当水工，后来就以水为姓。
    窦：大禹的后代，纪念先人。
    章：部落得一姓，其中就有章姓。
    云：黄帝的后代。
    苏：远古贵族昆吾，因为封在苏城为首领，就以地名为姓。
    潘：季孙的后代就以潘为姓。
    葛：最早的葛姓源于葛天氏。
    奚：奚姓是黄帝的子孙。
    范：贵族士会，被晋国国君封为范邑的首领，他的子孙就以地名为姓。
    彭：大彭国的创建者篯铿，就是彭祖。
    郎：地名为姓。
    鲁：周公旦的儿子伯禽受封于鲁地，后来鲁国的王公贵族都以鲁为姓。
    韦：韦姓的一支是韩信的后代。
    昌：始祖昌意，是黄帝的儿子。
    马：赵奢受封马服君，他的子孙姓马。
    苗：春秋时期的贲皇享用苗邑的物产，就以地名为姓。
    凤：黄帝的后代帝喾有个大臣叫凤鸟氏，主管历法天文，其后代以凤为姓。
    花：花姓人物最早的是花木兰，传说是南北朝人。
    方：神农氏的后代中有一位叫榆罔，其儿子受封方山后叫方雷，方雷的子孙就以地名为姓。
    俞：黄帝时，有个名医叫俞跗，是俞姓的始祖。
    任：黄帝的小儿子叫禺阳，受封于任邑。其子孙以任为姓。
    袁：周朝伯辕的后代将辕省去左半边，作为姓。
    柳：春秋时期展禽，改姓了柳下，史书上称他柳下季，柳姓由此开头。
    酆：周武王将弟弟封于酆邑，他的后代以地名为姓。
    鲍：春秋时期敬叔在齐国做官，受封于鲍邑，后代就以鲍为姓。
    史：黄帝有一名史官叫仓颉，后代以其官职为姓，就形成了史姓。仓颉被中国人尊为造字的圣人。
    唐：远古时，尧曾经在“唐”那个地方做首领。尧的族人就有以地名为姓。
    费：春秋时期，鲁国有个人物叫父，因有功劳受封为费邑的首领，现今山东鱼台县费亭一带，他的子孙就以地名为姓，世代延续。
    廉：远古黄帝的玄孙叫大廉，大廉的后代形成了廉姓。
    岑：周朝岑子建岑国，岑国的公族子孙，以国名为姓。
    薛：薛登的后代以薛为姓。
    雷：黄帝的一个妻子姓方雷，雷姓由此而来。
    贺：贺姓由庆姓转变来。
    倪：倪氏本为郳氏，因避仇改为倪。
    汤：汤王的后代有人就以祖先的名为姓。
    滕：周武王封弟弟叔绣封于滕国，后代子孙以国名为姓。
    殷：源出于子姓。
    罗：源于熊姓，是以国为名的姓。
    毕：毕公高的后代以国名为姓，称为毕氏。
    郝：源于子姓，是伏羲氏的后裔。
    邬：古帝颛顼后裔陆终氏第四子求言（妘姓）别封于邬，后代子孙以邬为姓。
    安：源自始祖。
    常：出自姬姓。
    乐：乐源出于子姓，西周末，公子衎字乐父，他的孙子以祖父字中“乐”命氏，称为乐氏。
    于：源出于姬姓。周武王的第三个儿子叔分封在邘国，与冯姓相同，后代去邑为姓，称为于氏。
    时：春秋时齐国贤士时子的后代以其名字中的“时”为姓。
    傅：黄帝裔孙大由封于傅邑，其后代以地名为姓。
    皮：樊仲皮的后人以他字中的“皮”为姓。
    卞：西周时，曹叔振铎的子孙分封在卞，他的后代称为卞氏。
    齐：齐康公后人，称为齐氏。
    康：源于谥号。
    伍：春秋时，楚庄王有个宠臣名叫伍参，是楚公族的后人。庄王封伍参为大夫，称为伍氏。
    余：春秋时，晋人由余穆公重用，他为穆公出谋划策，攻灭西戎十二国，使秦国成为西方霸主。他的后世子孙以其名字中的“余”为姓，称为余氏。
    元：春秋时，因卫国大夫元的封地在元，其后世子孙即以元为姓。
    卜：卜姓是古代从事占卜者的后代。
    顾：夏朝附庸国顾国，顾国为商汤所灭后，原王公族人以国名为姓。
    孟：春秋时，卫国国君叫卫襄公，卫襄公的儿子叫公孟。公孟的子孙有的姓公孟，有的省去公字而姓孟。
    平：战国末期，韩哀侯之子婼封于平邑。秦灭韩后，婼率族人迁居于下邑，他的孙子以原封地命姓。
    黄：出自赢姓，为陆终的后裔。
    和：祝融氏重黎的后人羲和的后代。
    穆：意为贤良、和气。
    萧：萧国后代子孙以国名为姓。
    尹：上古少昊氏的后裔。
    姚：舜因最初在姚墟，他的后裔子孙即以地为姓。
    邵：周文王第五子召公奭（音），食邑于召子孙遂以召为氏，后加邑为邵。
    湛：夏代早期，斟灌氏国后人以国为姓。
    汪：汪姓出自汪芒氏。
    祁：源自姬姓。
    毛：以封邑名作为姓的。
    禹：人名命名。
    狄：狄族的子孙后代就以族名作为自己的姓。
    米：春秋时代，楚国的后代当中有了米姓。
    贝：周文王的后代召公康，后代以国名为姓。
    明：古代部落首领燧人氏有一个重臣叫明由，他的后世子孙以其名作为姓。
    臧：鲁孝公儿子驱，他的后代以封地名作为自己的姓。
    计：禹后人的封地，后人便以国名。
    伏：伏姓出自风姓。
    成：出自姬姓。
    戴：宋戴公的后裔以他的谥号为姓。
    谈：出自子姓，是殷商皇族的后裔。
    宋：出自子姓，起源于战国时期。
    茅：周公旦的第三个儿子名叔，受封于茅邑，子孙以国名为姓。
    庞：出自高阳氏。黄帝的孙子颛顼生有八个儿子，其中有一个儿子名叫庞降，他的后人就以他的名字作为姓。
    熊：传说黄帝曾在有熊建都，所以黄帝的后代中有人以都城为姓。
    纪：出自姜姓。西周时，炎帝裔孙被封于纪，建立纪国，因属侯爵，所以被称作纪侯。纪国为齐国所灭，纪侯子孙便以国名为姓。纪姓另有一支由舒姓改成。
    舒：周朝封皋陶的后代于舒（今安徽庐江县西），建立舒国，曾被徐国所灭，后又复国，不久又亡于楚国，其国君族人以舒为姓。
    屈：是夏代屈骜的后代。
    项：春秋时期，楚国公子燕受封于项城（今河南项城县），建立项国。后项国被齐国所灭，其子孙以国名为姓。
    祝：周武王封黄帝的己姓后裔在祝国（今山东临沂县东南），后代子孙以国名为姓。
    董：相传黄帝后裔董父被舜帝封于鬷川，并赐以董姓，其子孙沿习为姓。
    梁：西周时，周宣王封秦仲二儿子康在夏阳梁山（今陕西韩城附近）立国为君，为梁康伯。后梁国为秦国所灭，其后代便以梁为姓。
    杜：舜封尧之子丹朱于唐。周成王时，唐国因不服王命被灭，成王改封唐国后人于杜（今西安市东杜陵）。宣王执政后，杜国君在朝任大夫，人称杜伯。杜伯因得罪了宣王的宠妃，被宣王屈杀。杜伯子孙大多逃往中原，留在杜城的一支以杜为姓。杜姓另一支为北魏时期鲜卑人独孤浑氏改姓。
    阮：出自偃姓。东夷族首领皋陶的后人在商朝时被封在泾水与渭水之间的阮国（今甘肃泾川县东南），商末时被周文王所灭，国人便以阮为姓。
    蓝：梁惠王三年，秦子向命封邑在蓝田，他的子孙以地名为姓。
    闵：源于姬姓。
    席：为籍姓所改。
    季：颛顼后裔陆终的第三个儿子为季连，他的后代以排行为姓。
    麻：春秋时期，熊婴做了齐国的大夫，他以祖先食邑为姓，改姓麻，史称麻婴。
    强：强姓出自姜姓，炎帝的后裔。
    贾：周朝康王时，封唐叔虞的小儿子公明于贾（今山西襄汾西南），称贾伯，他的后代以贾为姓。
    路：古代传说中黄帝后裔玄元，因有功，被封为路中侯，他的好几代子孙都延承这个爵称，后来渐渐变成了这个氏族的姓。他的后代以路为姓。
    娄：源于姒姓，是大禹的后代。周武王灭商后，把夏帝少康的后代东楼公封于杞国（河南杞县）。至春秋时期，杞国在周围大国的威胁下被迫东迁，杞君有一支子孙被封在娄邑（今山东诸城西南），后来以娄为姓。
    危：三苗族原居于河南南部至洞庭湖、鄱阳湖一带，三苗后裔以危为姓。
    江：出自嬴姓。颛顼裔孙伯益之后被封在江（今河南正阳西南），江国后来被楚国灭掉，国人便以江为姓。
    童：相传古帝颛顼有个儿子叫老童，他的后代以祖上名字为姓，称童氏。
    颜：古帝玄孙陆终第五子名安，周武王封其后裔挟于邾（今山东曲阜南陬村），后有邾武公名夷父，字颜，称之谓颜公。他的后代以祖字为姓，称颜氏。
    郭：姬序号为虢公，因虢与郭同音，转化变音，其后代便称为郭姓。
    梅：梅姓出于子姓，是商汤的后裔，发源于今天的安徽省亳州东南。
    盛：源出有三:出自姬姓，出自组名为氏，出自少数民族改姓。
    林：出自子姓，是商汤的后代。
    刁：出自姬姓。
    钟：伯宗的儿子伯州食邑在钟离，其后人以钟离作为姓，后简称钟。
    徐：出自赢姓，发源于今天的安徽省泗县。
    邱：出自姜姓，发源于今天的山东省昌乐。
    骆：出自姜姓。姜太公的后代有一个名叫公子骆，他的后世子孙以他的字作为姓，这样就有了骆姓。
    高：一支源于远古时代高台宫殿的建筑者，其始祖是黄帝时的大臣高元。
    夏：源于姒姓，是大禹的后裔。
    蔡：源自姬姓，发源于今天的河南省上蔡西南。
    田：西周初年，周武王封大禹的后代胡公满于陈（今河南省境内），并且建立了陈国，其后代都以陈作为姓。
    樊：出自姬姓，发源于今天的河南省济源县。
    胡：源于姬姓和归姓。
    凌：出自姬姓，发源于今天的河北省。
    霍：出自姬姓，发源于今天的山西省霍县西南。周武王在位时封叔武于霍，并且建立了霍国，世称叔武为霍叔，春秋时霍国被晋献公所灭，此后，霍国王室的后世子孙便以国名为姓。
    虞：最早出自姚姓，舜的儿子商均封于虞城，建立诸侯国虞国，其后世子孙便以国名作为自己的姓。
    万：出自姬姓，发源于今天的山西省芮城。
    支：来源比较复杂，而且模糊。传说尧舜时期有一个叫支父的人，他的子孙后代以支为姓。
    柯：出自姜姓，发源于齐国（今山东境内），是传说中炎帝的后裔，为姜子牙的嫡系子孙。
    昝：是由咎姓发展而来的。
    管：出自姬姓，发源于今天的河南省郑州市。
    卢：出自姜姓，发源于今天山东省的长清县。
    莫：出自高阳氏，发源于今天河北省任丘北部。
    经：春秋时，郑国公子共叔段被封于京（今河南荥阳县东南），其后代以封邑为姓。
    房：舜帝后来封丹朱的儿子陵于房国，陵的子孙便以地名为姓。
    裘：春秋时，卫国一大夫受封于裘，他的后代以封邑为姓，称裘氏。
    缪：古代“穆”与“缪”同音通用，所以春秋五霸之一的秦穆公也称作秦缪公，他的庶子以谥号命姓，称缪氏。
    干：春秋时，宋国有个大夫叫干，他的后代以他名字中的“干”字为姓，称干氏。
    解：出自姬姓，周朝贵族良的封地在解（今山西解县），他的子孙以地名为姓，称解氏。
    应：周武王克商后，封他的第四子于应（今河南鲁山县境内），他的后代以封地为姓，称应氏。
    宗：周朝时设有辅佐天子掌管宗室之事的春官，春秋时鲁国改称宗伯，负责宗庙祭祀礼仪，有世袭此职者，其后代便以“宗”为姓。
    丁：周朝时，姜子牙的儿子伋死后谥号为丁公，他的子孙以谥号为姓，称丁氏。
    宣：出自于谥号。
    贲：贲姓有十起源：一是来自鲁国县贲父的后代：一是来自戎族氏族：一是来自晋国苗贲父的后代：一是来自秦国嬴姓的后代：一是来自两周时期特殊称谓等等。贲还有一个姓的读音为，今不多见。
    邓：殷王武丁封他的叔父于邓国的曼城，称曼氏，后来又改封邓国。
    郁：古时有郁国，春秋时为吴国大夫的封邑，其国人多为郁姓。春秋时鲁国有宰相郁贡，他的后人以郁为姓，称郁氏。
    单：源出于姬姓。
    杭：源出于姒姓，是大禹的后代。
    洪：共工氏后人因先辈曾负责治水之职，便给共字加水旁，作为自己的姓。
    包：春秋后期，楚国有个大夫叫申包胥，他的子孙以他名字中的“包”字为姓。
    诸：春秋时，鲁国大夫受封于诸（今山东诸城西南），其后裔以封邑为姓，称诸氏。西汉时，历代名人有：无诸被封闽越王，他的后代以其名字中的“诸”为姓。五代后周人诸葛十朋，在赵匡胤陈桥兵变，黄袍加身后，不愿做宋朝臣民，隐居在会稽山。
    左：周朝史官有左、右之分，左史记言，右史记事。世袭左史之职的后人便以官职为姓。
    石：石厚的儿子石骀仲以祖父的字命姓，称石氏。
    崔：春秋时，齐丁公之子季子，把君位让给了兄弟叔乙，自己去了崔邑（今山东章丘），从此称为崔氏。
    吉：周宣王时有个大臣叫做尹吉甫，因赫赫军功而闻名，他的后代以他名字中的“吉”字为姓，称为吉氏。
    钮：《名贤氏族言行类稿》载，东晋有钮滔，其后为钮姓。
    龚：姬和，因封地被称为共（河南辉县）伯，子孙因国命姓，古时“共”、“龚”同音通用，后来传为龚
    程：颛顼有两个孙子重和黎，西周建立以后，重和黎的后代被封于程，并且建立了程国，后代以国为姓。
    嵇：嵇姓出自姒姓。
    邢：邢姓出自姬姓。
    滑：滑姓出自姬姓。
    裴：裴姓出自赢姓。
    陆：陆姓发源于今天的山东省平原县一带。
    荣：黄帝时，有一个叫荣援的人，黄帝封他为诸侯，并且建立了荣国，他曾经奉黄帝之命铸钟十二个，其后世子孙都以荣为姓。
    翁：在夏朝初年，有一位贵族叫翁难乙，他可以说是翁姓的始祖。
    荀：荀姓出自姬姓。
    羊：西周有一个种官职叫羊人，担任这个官职之人的后世子孙，就以此为姓，这样就有了羊姓。
    於：发源于今天的河南省。
    惠：远古时期的部落联合首领颛顼的后代叫惠连，他的后代子孙就以“惠”作为自己的姓，这样就有了惠姓。
    甄：上古部落首领皋陶的次子名仲甄，在夏朝时为卿仕，受封于甄，他的后裔也取姓为甄。
    麴：麴本是酿酒的主要原料，西周时成为掌管酿酒业的官名，世袭此职的人便以麴为姓。
    家：源自于姬姓。
    封：炎帝的后代，以地为姓。
    芮：源于姬姓。周武王封姬良夫于芮，后人以国为姓。
    羿：为夏朝著名弓箭射手后羿的后代。
    储：相传上古时有储国，储国人的后代以储为姓。
    靳：战国时期靳尚的后代便以靳为姓。
    汲：源自于姬姓。春秋时期，卫宣公的公子居住的地方叫汲（今河南汲县），他的后代以汲为姓。
    邴：齐大夫邴的封地也叫邴（今山东费县东南），他的后代以封邑为姓，也称邴氏，也有去邑成姓，称丙氏。
    糜：最早始于夏代，是以谷物名称命名的姓。
    松：起源于秦朝。
    井：源自姜姓。
    段：春秋时，老子李聃的孙子李宗在晋国为官，封于段干，他的后代一支姓段。
    富：周襄王时有姬姓大夫名富辰，直言敢谏，不避贵胄，为人称道，他的后代有的以其名字为姓，称富氏。
    巫：担任巫官后人有的以巫字为姓。
    乌：出自金天氏。
    焦：神农氏的后人，以国为姓。
    巴：战国中期，巴国被秦国攻灭，巴人便以国名为姓。
    弓：来源为职官名。
    牧：力牧助黄帝治理天下，立有大功，他的后代便以祖上名字为姓，一支姓力，一支姓牧。
    隗和：汤灭夏桀后，封其后人于隗，建大隗国，春秋时为楚国所灭，其子孙以国名为姓，称隗氏。春秋时晋国大举进攻赤狄诸部落，赤狄人后裔汉化后沿用隗姓。
    山：周朝掌管山林的官员叫山师，子孙便称为山氏。
    谷：源自于嬴姓。
    车：春秋时，秦国大夫子车仲的后代为车姓。
    侯：春秋时，晋哀侯和缗侯均为晋武公所杀，哀侯和缗侯的子孙逃难他国后，便以祖上受封爵位为姓，称侯氏。
    宓或：音时是安静的意思。读，古同“伏”，“伏羲”亦作“宓羲”。
    蓬：源自姬姓。
    全：泉府官的后人以职官为姓，便有了泉姓，又因“全”、“泉”同音通用，于是也便有了全姓。
    郗：苏忿生的后代受封于郗（今河南沁阳），其后人便以封邑命姓为郗姓。
    班：春秋时，楚国若敖的孙子出生后曾被母亲弃于野外，吃虎乳长大。因虎有斑纹，其后代子孙以此事件命姓，为班姓。
    仰：仰姓一支出自舜帝时大臣仰延之后。
    秋：秋姓为少昊氏之后。
    仲：商汤王左相仲虺的后代和春秋时宋庄公之子（字子仲）的后代，也以祖上名字为姓。
    伊：相传尧出生在伊祁山，他的后代有的以伊为姓。
    宫：周朝时有专门负责宫廷修缮和清洁事宜的官，名宫人，世袭此职的后人便以宫为姓。
    宁：出自赢姓，春秋时期，秦国国君秦襄公的谥号为“宁”，他的后世子孙当中有的人就以先人的谥号作为姓。
    仇：出自九吾氏。
    栾：出自姬姓，发源于河北栾城一带。
    暴：最初形成于商朝，发源地在今天河南省境内。
    甘：夏朝的诸侯国当中有一个甘国，后代以国为姓。
    钭：出自姜姓。齐国的君主康公就以酒器名“钭”作为自己的姓，这样就有了钭姓。
    厉：出自姜姓。
    戎：出自姜姓。在西周时期，有一个戎国，它是齐国的附庸，后来戎国灭亡了，其公族便以国名作为了姓。
    祖：汤是商朝的开国君主，他的后人祖乙、祖辛、祖丁等都先后做了商王，其支庶子孙以“祖”作为姓。
    武：发源于今天广西省的水淳县北部。
    符：出自姬姓。
    刘：远古尧帝的子孙中，有个叫刘累的，据说他能驯化龙和驾龙飞奔，他就是刘姓始祖。
    景：在春秋时期景差的后世子孙便以景作为自己的姓，这样就有了景姓。
    詹：据传说“五帝”之一的舜，曾经封黄帝后裔中得詹（受尊敬）的人为詹姓，这样就有了詹姓。
    束：战国时期齐国有一疏姓部族，后改为束姓。
    龙：舜做部落联合首领，他的手下有一大臣名纳，为龙氏，他的后世子孙都姓龙。
    叶：发源于今天河南省的叶县。
    幸：起源据与幸臣有关。
    司：部落首领神农氏的属下当中，有专门负责占卜的大臣名司怪，他的后世子孙便以“司”作为姓。
    韶：乐官的后世子孙以其祖上所作乐曲的曲名作为自己的姓，这样就有了韶姓。
    郜：出自姬姓。
    黎：发源于今天河南省的浚县。
    蓟：发源于今天的北京市西南。
    薄：薄国君主的族人，便以封国之名作为自己的姓。
    印：源自于姬姓。春秋时期，郑穆公之子字子印，子印的孙子名段，在郑国为卿大夫，以其祖父的字中的“印”为姓，称为印氏。
    宿：伏羲氏的后裔被封在宿国（今山东东平），其后人以国名为姓。
    白：白阜被尊为白姓始祖。
    怀：是无怀氏的后人。
    蒲：舜的后代封于蒲（今山西永济），舜的后代因此以地名为姓。
    邰：帝尧任命弃大司农，封于邰（今陕西武功），其子孙以地名“邰”为姓。
    从：从姓源自于姬姓。
    鄂：春秋时，晋国大夫顷父之子嘉父叛晋，奔至鄂，称为鄂侯，他们的后代以鄂为姓。
    索：索姓是汤王的后裔。
    咸：帝喾为部落首领时，有个大臣叫咸邱黑，是咸姓的始祖。
    赖：是炎帝的后裔。
    卓：是春秋时楚国王族的后裔。
    蔺：是春秋时晋国王族的后裔。
    屠：古人有以屠宰为业者，他们的后代以屠为姓。
    蒙：颛顼的后裔。
    池：池姓源于嬴姓。
    乔：黄帝后代以山名取为桥姓，后来有的简化为乔姓。
    阴：周朝有阴邑，其大夫的后裔以食邑为姓，称阴氏。
    郁：相传大禹的老师叫郁华，他便是郁姓的始祖，其后裔以祖字为姓，称郁氏。
    胥：胥是以祖辈的名字命名的姓，起源于华胥、赫胥二氏。
    能：周成王时，大臣熊绎因功受封为男爵，他的后代将熊姓去四点改为能姓。
    苍：相传黄帝有二十五个儿子，其中有个叫苍林，他的后代以祖辈名字为姓，称苍氏。黄帝的孙子颛顼有八个儿子，都是治世能臣，世称“八恺”，其中长子名苍舒，他的后代也以祖字为姓，形成苍姓一支。大约汉代以后，苍姓改为仓姓。
    双：双姓与蒙姓同出一宗，是颛顼的后裔。夏朝建立后，封颛顼之后于蒙双，有子孙以地名为姓，成为蒙姓和双姓两支。
    闻：出自复姓闻人氏。
    莘：莘姓一族是古帝祝融的后代。
    党：党姓出自夏朝，居住在党项的王族多姓党。
    翟：翟姓出自隗姓。
    谭：谭姓是颛顼的后代。
    贡：源于端木氏。
    劳：劳姓起源于西汉时期。
    逄：逄姓出自姜姓。
    姬：相传黄帝的降生地有一条叫做“姬”的河，于是便以“姬”作为姓。
    申：申姓发源于今天河南省的南阳。
    扶：大禹任部落联盟首领的时候，其属下有扶登氏，他就是扶姓的始祖。
    堵：堵姓出自姬姓。
    冉：帝喾为炎帝部落首领时，属下的八个部落当中就有冉姓，其后人世代姓冉。
    宰：宰姓源于姬姓，春秋时期，周公孔曾经在周朝任太宰，人称宰孔，其后世子孙就以“宰”作为了姓。
    郦：郦姓出自轩辕氏。
    雍：雍姓出自轩辕氏，发源于今天河南省的内黄。
    郤：春秋时期，晋国公族子弟叔虎建立郤国，后世子孙以封地名作为姓。
    璩：古时候有一个叫蘧邑的地方，那里的人最初是以地名“蘧”作为姓的，后来蘧姓改成了璩姓。
    桑：桑姓出自金天氏。
    桂：桂姓出自姬姓。
    濮：舜任部落首领时，其子孙散封于濮地，这些人的后代便以地名作为姓。
    牛：周朝牛父的子孙以他的字“牛”作为自己的姓。
    寿：是彭祖的后裔。
    通：春秋时，巴国有大夫受封于通川，其后世子孙以封地名“通”作为姓。
    边：商代有诸侯国边国，国君为伯爵，被称为边伯，其后代称为边氏。
    扈：扈姓为大禹的后代。
    燕：南燕灭国后，伯倏的后人以国名为姓。
    冀：相传唐尧的后代被周武王分封在冀（今山西河津），其国人便以冀为姓。
    郏：周朝时，有地名称为郏鄏（今河南洛阳西）。居住在这里的人以地名为姓。
    浦：出自于姜姓。
    尚：尚姓一支为姜太公的后裔。
    农：农姓出自神农氏，是炎帝的后裔。
    温：周武王的儿子叔虞的后代被封于温（今河南温县），其后以封地为姓。
    别：《姓苑》记载别氏，京兆人，望出天水、京兆”等。
    庄：春秋时宋戴公名武庄，他的后人以其名字中的“庄”为姓。
    晏：黄帝后裔颛顼的孙子名陆终，他的儿子为晏安，其后代遂为晏姓。
    柴：柴姓出自姜姓。
    瞿：瞿姓一支源于人名，商代有一大夫，因封地在瞿上而得名瞿父，他的子孙以祖先名为姓。
    阎：上古周武王建立周朝后，武王封一位王族仲奕于阎乡，仲奕的后代以祖上封邑为姓，称阎氏。
    充：充人后代以职官名为姓。
    慕：慕姓出自慕容氏。
    连：黄帝孙子颛顼的曾孙陆终有子名惠连，他的后代以祖先名中的字为姓。
    茹：茹茹以族名为姓。
    习：古代有诸侯国习国，其国君后世子孙以习为姓。
    宦：宦姓非常罕见。
    艾：艾姓源于夏代，是禹王的后代。
    鱼：子鱼孙辈嫡传一支以祖父名字为姓，称鱼氏，支庶孙以“鱼孙”为姓。
    容：黄帝有史臣容成，其后代以容字为姓。
    向：炎帝有一个名叫向的裔孙被封为诸侯，其后代子孙便以先人的名为姓。
    古：古姓起源于殷商时期。
    易：易姓形成于春秋时期。
    慎：慎姓发源于今天安徽省的颖上县西北。
    戈：夏代东夷有寒国的儿子在戈国，其后代子孙就以国名作姓。
    廖：廖姓起源于今天的河南省唐河县西。
    庾：相传在远古帝尧时代，有掌庾大夫。以官名作为姓，这样就有了庾姓。
    终：终姓是黄帝的后裔。
    暨：暨姓是颛顼的后人。
    居：居姓出自杜姓。
    衡：出自伊姓。
    步：步姓发源于今天山西省的临汾市南。
    都：春秋初年，郑国有个公族大夫叫子都，他的子孙以其字“都”作为姓。
    耿：出自姬姓。
    满：西周胡公满的支庶子孙当中有人以先祖名字中的“满”字作为姓。
    弘：在春秋时期，卫国有个大夫叫弘演，他的后世子孙就以其名字中的“弘”字作为自己的姓。
    匡：春秋时期，鲁国大夫施孝权的家臣句须任匡邑宰，后来他的孙子就以祖父做官地的名称作为自己的姓。
    国：大禹为首领时，有一个专门掌管车马的人名叫国哀，后世子孙都随之以国为姓。
    文：周武王追谥父亲西伯姬昌为周文王，子孙以其谥号“文”作为了姓。
    寇：春秋时期卫灵公的孙子在卫国任司寇，他的后世子孙就以官名“司寇”作为自己的姓，后来简称寇氏。
    广：上古广成子的后代以广成为姓。
    禄：出自子姓，是殷商最后一个帝王纣王的后代。
    阙：阙姓起源于春秋之时。
    东：东姓源于远古之时。
    欧：春秋时，越王无疆将次子封于乌程（今浙江吴兴）欧余山的南面，古代称山南为阳，所以他被称为欧阳亭侯。
    殳：出于有虞氏。
    沃：沃丁的子孙有的以他的名字为姓，称沃氏。
    利：出于理姓，与理、李、里同宗。
    蔚：北周时设有蔚（）州，居住在这里的人以地名为姓。
    越：越国公族支庶子孙有的以原国名为姓，形成越姓。
    夔：远古时，尧和舜的乐官名叫夔，此姓的起源就在尧舜帝时代。夔原指龙形动物。
    隆：隆姓源于地名。春秋时，鲁国境内有隆邑（今山东泰安西南），居住在这里的人以其地名为姓，形成隆姓。
    师：源于技艺和职官名。
    巩：周敬王时有同族卿士简公，受封于巩（今河南巩县），称为巩简公。巩简公的子孙以封邑为姓，称巩氏。
    厍：厍姓源于北周厍狄氏所改姓。
    聂：春秋时，齐丁公封其支庶子孙于聂邑（今山东茌平西），建聂国，为齐国附庸。其后世子孙以国名为姓，形成聂姓。
    晁：源于姬姓。古代“朝”与“晁”相通，都有早晨之意，所以在姓中“朝”也写作“晁”。周朝时，景王之子王子朝的子孙以朝为姓，又称“晁”氏。
    勾：又写作“句”。其姓最早见于《山海经》：“困民之国，勾姓”。
    敖：上古五帝之一颛顼的老师叫做大敖，有的写做太敖，他的后代以祖上名字为姓，形成敖姓。
    融：融姓起源于上古，是颛顼高阳氏后裔。
    冷：冷姓系泠姓所改。
    訾或：周朝时有訾邑（今河南巩县西南），居住在这里的人有的以地名为姓。
    辛：辛姓源于姒姓。
    阚：阚姓源自姜姓。
    那：商朝君王武丁的裔孙到那地（今湖北荆门东南）定居，形成那姓。
    简：春秋时，晋国大夫狐鞫居为周文王的后裔，他被封于续，死后谥号为简。
    饶：饶姓一支源于姜姓。
    空：空姓为复姓空同、空桑、空相等省文改成。
    曾：曾参是春秋时的鲁国人，而且是孔子的得意弟子。
    毋：毋姓起源于上古，尧属下毋句制造出乐器罄，他就是毋姓的始祖。
    沙：炎帝做部落首领时，他的属下有夙沙氏，其后世子孙省文为沙氏。
    乜：产生于春秋时期，卫国有大夫的采邑在乜城，其后人就把采邑名作为了姓。
    养：春秋时期，吴国公子掩余、烛庸逃到楚国，楚王把养地（今河南境内）封给了他们，后来其子孙后代就以祖上的封地名“养”作为姓。
    鞠：鞠姓出自姬姓，周始祖弃有支孙名叫陶，他出生时手心有纹似“鞠”字形，所以取名鞠陶，他的后世子孙便以鞠作为自己的姓。
    须：须姓源自风姓，是太昊伏羲氏的后代，春秋时期有风姓国叫须句国，该国的国民则称须句氏，后来改为须氏。
    丰：春秋时期，郑穆公的儿子丰，在郑僖公时任大夫，他的孙子便以其名“丰”作为姓，而称为丰施、丰卷，后来他们的子孙也随之姓丰。
    巢：尧为帝时，有大臣巢父常居于山中，以树为巢，故称有巢氏。禹帝时曾封有巢氏之后建立巢国（今安徽境内）。在夏、商、周三代，巢国世代为诸侯国，春秋时期，楚国灭掉巢国，巢国后代公族便以原国名作为姓。
    关：夏朝时，夏桀是一个荒淫的暴君，有个叫关龙逢的贤臣曾去劝谏他，结果被处死，关龙逢的后代便姓关。
    蒯：商朝时有蒯国（在今天河南省洛阳市一带），其国人后来以国名作为姓。
    相：夏朝有一个帝王叫相，他的后裔支庶子孙，有的就以祖上的名字作为姓。
    查：查姓出自姜姓，是炎帝后裔。
    後：後姓出自太昊氏，是上古东夷部族首领太昊的孙子後照的后代。
    荆：西周初年，荆君有庶出子孙则以原国号作为姓。
    红：红姓出自熊姓。
    游：春秋时期，郑国国君郑穆公之子名偃，字子游，他的孙子以祖父的字作为姓。
    竺：汉代，有枞阳人竹晏改“竹”为“竺”，其后世子孙即沿用竺姓。
    权：商武帝武丁的后人被封于权地（今湖北境内），其后代以国为姓。
    逯：秦国公族大夫有人受封于逯邑（今陕西境内），其后世子孙以地位姓。
    盖：春秋时期公族大夫受封于盖邑（今山东沂水县西北），后代子孙就以地位姓。
    益：伯益是秦国的始祖，其支庶子孙中有人以祖上的名字作为自己的姓。
    桓：源于上古，黄帝时有大臣桓常，他是桓姓的始祖。
    公：出自姬姓。
    万俟：东晋时，鲜卑族的万俟部落随拓跋氏其部落族人以“万俟”为姓。
    司马：相传帝少昊始设司马一职，掌管军政。
    上官：春秋时期，楚庄王幼子名子兰，担任上官大夫。子兰的子孙便以地名命姓。
    欧阳：蹄被封于乌程欧余山的南面，被称为欧阳亭侯，他的支庶子孙以封地命姓。
    夏侯：杞简公的弟弟被称为夏侯，他的子孙于是以“夏侯”为姓。
    诸葛：商朝时，伯夷的后裔。
    闻人：春秋时少正卯声誉高被称做“闻人”。他的支庶子孙有的便以“闻人”为姓。
    东方：伏羲氏创八卦，八卦以东方为尊，其后代支庶子孙便以东方为姓。
    赫连：是西汉时匈奴族的姓，始祖为匈奴右贤王刘去卑。
    皇甫：春秋时宋戴公之子公子充石，字皇父。他的孙子以祖父之字为姓。
    尉迟：为鲜卑族姓。
    公羊：春秋时鲁国人公孙羊孺颇有才华，他的支庶子孙以祖上名字中的两个字命姓，形成公羊姓。公羊姓今已融入公姓。
    澹台：春秋时期，孔子有个弟子名叫澹台灭明，字子羽。
    公冶：春秋时鲁国季氏有族人叫季冶，字公冶，他的子孙便以公冶为姓。
    宗政：也作“宗正”。宗正是秦汉时期的官名，专管皇帝亲属的有关事宜，自秦始皇时开始设置。汉朝时。高祖刘邦有个本家亲戚担任宗正这一官职，后改姓为宗正。
    濮阳：濮阳为地名，在今河南濮阳县，当地人有以地名为姓的，称为濮阳氏。
    淳于：春秋时期有淳于国，淳于国的人后来称为淳于氏。
    单于：历史上匈奴族的最高首领称为“单于”，他们的后代中有以“单于”为姓的。
    太叔：春秋时，卫文公有儿子名叫太叔仪，他的后代就是太叔氏。
    申屠：为上古舜帝的后代。
    公孙：周朝国君一般立正妻所生的长子为太子，其他儿子就称为公子。公子的儿子被称为公孙。有公孙的后代就以公孙为自己的姓。
    仲孙：出自姬姓。
    轩辕：相传黄帝号轩辕氏，他的后代中有以轩辕为姓的。
    令狐：魏颗被晋君别封于令狐，“以邑为氏”而姓的令狐。
    钟离：源出于嬴姓。
    宇文：鲜卑族有个部落首领自称“字文氏”。
    长孙：北魏建立后，拓跋硅认为大伯沙莫雄的大儿子嵩是曾祖父的长孙，就赐嵩为“长孙”氏。嵩的后代也就一直以长孙为自己的姓。
    慕容：三国时期，鲜卑族首领莫护跋在棘城以北建立国家，其后代以国为姓。
    鲜于：源出子姓。
    闾丘：春秋时齐国有大夫名叫闾丘婴，他的后代称为闾丘氏。
    司徒：先秦设司徒这一官职。有的司徒后代就把官职作为自己的姓，称为司徒氏。
    司空：出自姒姓，是大禹的后代，以官职名为姓。
    佟：源于妫姓，出自夏王朝末期太史终古的后代。
    第五：源于妫姓，出自汉高祖刘邦强迁原战国时期齐国的田氏公族时封的编序，属帝王赐姓。
    言：出自春秋时言偃之后，以祖名为姓。
    福：出自春秋时齐国大夫福子丹之后，以祖名为姓。
""".strip().split()
SurNameMessage = {}

for line in _message:
    key, value = line.split('：', maxsplit=1)
    SurNameMessage[key] = value

HundredFamilyNames = """
赵	钱	孙	李	周	吴	郑	王	冯	陈	褚	卫	蒋	沈	韩	杨	朱	秦	尤	许
何	吕	施	张	孔	曹	严	华	金	魏	陶	姜	戚	谢	邹	喻	柏	水	窦	章
云	苏	潘	葛	奚	范	彭	郎	鲁	韦	昌	马	苗	凤	花	方	俞	任	袁	柳
酆	鲍	史	唐	费	廉	岑	薛	雷	贺	倪	汤	滕	殷	罗	毕	郝	邬	安	常
乐	于	时	傅	皮	卞	齐	康	伍	余	元	卜	顾	孟	平	黄	和	穆	萧	尹
姚	邵	湛	汪	祁	毛	禹	狄	米	贝	明	臧	计	伏	成	戴	谈	宋	茅	庞
熊	纪	舒	屈	项	祝	董	梁	杜	阮	蓝	闵	席	季	麻	强	贾	路	娄	危
江	童	颜	郭	梅	盛	林	刁	钟	徐	邱	骆	高	夏	蔡	田	樊	胡	凌	霍
虞	万	支	柯	昝	管	卢	莫	经	房	裘	缪	干	解	应	宗	丁	宣	贲	邓
郁	单	杭	洪	包	诸	左	石	崔	吉	钮	龚	程	嵇	邢	滑	裴	陆	荣	翁
荀	羊	於	惠	甄	麴	家	封	芮	羿	储	靳	汲	邴	糜	松	井	段	富	巫
乌	焦	巴	弓	牧	隗	山	谷	车	侯	宓	蓬	全	郗	班	仰	秋	仲	伊	宫
宁	仇	栾	暴	甘	钭	厉	戎	祖	武	符	刘	景	詹	束	龙	叶	幸	司	韶
郜	黎	蓟	薄	印	宿	白	怀	蒲	邰	从	鄂	索	咸	籍	赖	卓	蔺	屠	蒙
池	乔	阴	郁	胥	能	苍	双	闻	莘	党	翟	谭	贡	劳	逄	姬	申	扶	堵
冉	宰	郦	雍	舄	璩	桑	桂	濮	牛	寿	通	边	扈	燕	冀	郏	浦	尚	农
温	别	庄	晏	柴	瞿	阎	充	慕	连	茹	习	宦	艾	鱼	容	向	古	易	慎
戈	廖	庾	终	暨	居	衡	步	都	耿	满	弘	匡	国	文	寇	广	禄	阙	东
欧	殳	沃	利	蔚	越	夔	隆	师	巩	厍	聂	晁	勾	敖	融	冷	訾	辛	阚
那	简	饶	空	曾	毋	沙	乜	养	鞠	须	丰	巢	关	蒯	相	查	後	荆	红
游	竺	权	逯	盖	益	桓	公	万俟	司马	上官	欧阳	夏侯	诸葛
闻人	东方	赫连	皇甫	尉迟	公羊	澹台	公冶	宗政	濮阳
淳于	单于	太叔	申屠	公孙	仲孙	轩辕	令狐	钟离	宇文
长孙	慕容	鲜于	闾丘	司徒	司空	亓官	司寇	仉	督	子车
颛孙	端木	巫马	公西	漆雕	乐正	壤驷	公良	拓跋	夹谷
宰父	谷梁	晋	楚	闫	法	汝	鄢	涂	钦	段干	百里	东郭	南门
呼延	归	海	羊舌	微生	岳	帅	缑	亢	况	后	有	琴	梁丘	左丘
东门	西门	商	牟	佘	佴	伯	赏	南宫	墨	哈	谯	笪	年	爱	阳	佟
第五	言	福	百 家 姓 终
""".strip().split()
if __name__ == '__main__':
    print(SurNameMessage)
    print(SurName)
    print(HundredFamilyNames)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/7/4 14:39
# @Author: Jtyoui@qq.com
import datetime
import re


def plus_date(start: str, end: str, format_='%Y-%m-%d %H:%M:%S') -> datetime:
    """两个日期相减得差数 :end-start
    :param start: 一个正确的日期
    :param end: 另一个正确的日期
    :param format_: 日期格式化
    :return: 被加后的日期
    """
    if '%H:%M:%S' in format_:
        if ':' not in start: start += ' 00:00:00'
        if ':' not in end: end += ' 00:00:00'
    start = datetime.datetime.strptime(start, format_)
    end = datetime.datetime.strptime(end, format_)
    plus = end - start
    return plus


def add_day(date: str, format_='%Y-%m-%d', to_str=False, **kwargs) -> datetime:
    """一个日期加多少天或者时间
    :param date: 一个正确的日期
    :param format_: 日期格式化
    :param to_str: 是否打印字符串
    :param kwargs: 可以填写 :days=xx，seconds=xx,microseconds=xx,milliseconds=xx,minutes=xx,hours=xx,weeks=xx
    :return: 被加后的日期
    """
    start = datetime.datetime.strptime(date, format_)
    end = datetime.timedelta(**kwargs)
    if to_str:
        add = start + end
        return add.strftime(format_)
    return start + end


def check_date_type(date_str: str) -> (bool, datetime):
    """验证数字字符串是否日期类型：例如 2018-2-1
    :param date_str: 日期类型 例如 2019-2-1，返回真、2019-2-29 返回假（2019年没有2月29号）。
    :return: 返回元组，（真、datetime类型）、（假、None）
    """
    try:
        p = "[!#$%&*+,-./:;=?@^_` |~年月日号点分秒]"
        data = []
        for r in re.split(p, date_str):
            if r and isinstance(r, str):
                data.append(int(r))
        d = datetime.datetime(*data)
        return True, d
    except:
        return False, None


if __name__ == '__main__':
    p = plus_date(start='2018-02-01', end='2018-01-01')
    print(p.days)
    print(add_day(date='2018-02-28', days=1, to_str=True))
    print(check_date_type('2018-2-28 23:00:00'))
    print(check_date_type('2018/2/1'))
    print(check_date_type('2018年2月1日14点15分20秒'))



from setuptools import setup, find_packages
from jtyoui import __version__, __author__, __description__, __email__

with open('README.md', encoding='utf-8') as f:
    long_text = f.read()

setup(
    name=__author__.lower(),
    version=__version__,
    description=__description__,
    long_description=long_text,
    long_description_content_type="text/markdown",
    url='https://github.com/jtyoui/Jtyoui',
    author=__author__,
    author_email=__email__,
    license='MIT Licence',
    packages=find_packages(),
    platforms=["window10", "Linux"],
    package_data={'': ['*']},
    install_requires=['requests', 'click'],
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    zip_safe=False,
    entry_points={"console_scripts": ["jtyoui = jtyoui.cli:main"]}
)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/11/28 14:58
# @Author: Jtyoui@qq.com
import json


def content_type(requests):
    """根据不同的content_type来解析数据"""
    if request.method == 'POST':
        if requests.content_type == 'application/x-www-form-urlencoded':
            data = requests.form
        elif requests.content_type == 'application/json':
            data = request.json
        else:  # 无法被解析出来的数据
            data = json.loads(requests.data)
        return data
    elif request.method == 'GET':
        return request.args


if __name__ == '__main__':
    from flask import Flask, request, jsonify

    app = Flask(__name__)


    @app.route('/hi', methods=['POST', 'GET'])
    def hello():
        data = content_type(request)  # 所有的请求信息
        return jsonify(data=data)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/7/30 10:33
# @Author: Jtyoui@qq.com
import tensorflow
from tensorflow.contrib.rnn import LSTMCell
from tensorflow.contrib.crf import crf_log_likelihood, viterbi_decode


class NerCRF:
    batch_size = 64
    epoch_num = 20
    hidden_dim = 300
    embeddings = 300
    dropout_keep_prob = 0.5
    lr = 0.001
    clip_grad = 5.0
    model_path = './model/'
    label_tag = {"O": 0, "B-a": 1, "I-a": 2, "B-b": 3, "I-b": 4, "B-c": 5, "I-c": 6}  # 必须重写
    num_tags = len(label_tag)
    vocab = {}  # 必须重写

    def get_feed_dict(self, seq_, label_):
        max_len = max(map(lambda x: len(x), seq_))
        seq_list, seq_len_list, label_list = [], [], []
        for seq, label in zip(seq_, label_):
            bu = [0] * max(max_len - len(seq), 0)
            seq_list.append(seq[:max_len] + bu)
            label_list.append(label[:max_len] + bu)
            seq_len_list.append(min(len(seq), max_len))
        feed_dict = {word_ids: seq_list, seq_length: seq_len_list, labels: label_list, lr_pl: self.lr,
                     dropout_pl: self.dropout_keep_prob}
        return feed_dict

    def read_data(self):
        """必须重写该方法，格式要保证一致"""
        word = [[1, 2, 3], [2, 3, 4, 5], [5, 6, 7, 3, 5, 2]]
        word_label = [[0, 5, 6], [3, 4, 0, 0], [1, 2, 0, 0, 3, 4]]
        return word, word_label

    def train(self):
        saver = tensorflow.train.Saver()
        with tensorflow.Session() as sess:
            sess.run(tensorflow.global_variables_initializer())
            for epoch in range(self.epoch_num):
                for _ in range(self.batch_size):
                    feed = self.get_feed_dict(*self.read_data())
                    _, loss_train, step_num_ = sess.run(fetches=[train_op, loss, global_step], feed_dict=feed)
                    print('loss', loss_train)
            saver.save(sess, self.model_path, global_step=step_num_)

    def test(self, sentence):
        feed_dict = dict()
        saver = tensorflow.train.Saver()
        with tensorflow.Session() as sess:
            cp = tensorflow.train.get_checkpoint_state(checkpoint_dir=self.model_path)
            saver.restore(sess, save_path=cp.model_checkpoint_path)
            label = [0.0] * len(sentence)
            feed_dict[word_ids] = sentence
            feed_dict[seq_length] = [len(sentence)]
            feed_dict[labels] = [label]
            feed_dict[lr_pl] = self.lr
            feed_dict[dropout_pl] = self.dropout_keep_prob
            tag, transition = sess.run(fetches=[logic, transition_params], feed_dict=feed_dict)
            tags = []
            for t in tag[0]:
                [vite], _ = viterbi_decode(t[:len(sentence)], transition_params)
                tags.append(vite)
            print(tags)
        return tags


word_ids = tensorflow.placeholder(dtype=tensorflow.int32, shape=[None, None])
labels = tensorflow.placeholder(dtype=tensorflow.int32, shape=[None, None])
seq_length = tensorflow.placeholder(dtype=tensorflow.int32, shape=[None])
dropout_pl = tensorflow.placeholder(dtype=tensorflow.float32, shape=[])
lr_pl = tensorflow.placeholder(dtype=tensorflow.float32, shape=[])

embedding_mat = tensorflow.random.uniform(shape=(len(NerCRF.vocab), NerCRF.embeddings), minval=-0.25, maxval=0.25)

with tensorflow.variable_scope("words"):
    _word_embeddings = tensorflow.Variable(embedding_mat, dtype=tensorflow.float32, trainable=True)
    word_embeddings = tensorflow.nn.embedding_lookup(params=_word_embeddings, ids=word_ids, name="word_embeddings")
    word_embeddings = tensorflow.nn.dropout(word_embeddings, dropout_pl)

with tensorflow.variable_scope("bi-lstm"):
    cell_fw = LSTMCell(NerCRF.hidden_dim)
    cell_bw = LSTMCell(NerCRF.hidden_dim)
    (output_fw, output_bw), _ = tensorflow.nn.bidirectional_dynamic_rnn(cell_fw=cell_fw, cell_bw=cell_bw,
                                                                        inputs=word_embeddings,
                                                                        sequence_length=seq_length,
                                                                        dtype=tensorflow.float32)
    output = tensorflow.concat([output_fw, output_bw], axis=-1)
    output = tensorflow.nn.dropout(output, dropout_pl)

with tensorflow.variable_scope("project"):
    W = tensorflow.get_variable(name="W", shape=[2 * NerCRF.hidden_dim, NerCRF.num_tags],
                                initializer=tensorflow.contrib.layers.xavier_initializer(),
                                dtype=tensorflow.float32)
    b = tensorflow.get_variable(name="b", shape=[NerCRF.num_tags], initializer=tensorflow.zeros_initializer(),
                                dtype=tensorflow.float32)
    s = tensorflow.shape(output)
    output = tensorflow.reshape(output, [-1, 2 * NerCRF.hidden_dim])
    pred = tensorflow.matmul(output, W) + b
    logic = tensorflow.reshape(pred, [-1, s[1], NerCRF.num_tags])

log_likelihood, transition_params = crf_log_likelihood(inputs=logic, tag_indices=labels,
                                                       sequence_lengths=seq_length)
loss = -tensorflow.reduce_mean(log_likelihood)

with tensorflow.variable_scope("train_step"):
    global_step = tensorflow.Variable(initial_value=0, trainable=False)
    optimizer = tensorflow.train.AdamOptimizer(learning_rate=lr_pl)
    grads_and_vars = optimizer.compute_gradients(loss)
    grads_and_vars_clip = [[tensorflow.clip_by_value(g, -NerCRF.clip_grad, NerCRF.clip_grad), v] for g, v in
                           grads_and_vars]
    train_op = optimizer.apply_gradients(grads_and_vars_clip, global_step=global_step)

if __name__ == '__main__':
    ner = NerCRF()
    ner.train()



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/5/29 17:44
# @Author: Jtyoui@qq.com

"""汉语拼音纠错（Chinese error correction）"""
from jtyoui.language.PinYin import load_pin_yin, chinese_to_pin_yin
from jtyoui.data import fuzzy_tone
from collections.abc import Iterable
import os


class ChineseError:
    """基于拼音谐音纠错"""

    def __init__(self, words_or_file):
        self._model = load_pin_yin(False)  # 加载拼音模型
        self._words = {}
        if (not isinstance(words_or_file, str)) and isinstance(words_or_file, Iterable):
            for word in words_or_file:
                ls = chinese_to_pin_yin(self._model, word)
                self._words[word] = ' '.join(ls)
        elif os.path.exists(words_or_file):
            with open(words_or_file, encoding='utf-8') as f:
                for line in f:
                    ls = chinese_to_pin_yin(self._model, line)
                    self._words[line] = ' '.join(ls)
        else:
            raise TypeError('输入一个纠错列表或者文件地址')
        self.fuzzy_tone = fuzzy_tone
        self._word = None

    def _flag(self, ls):
        total, index = [], -1
        value = ' '.join(ls)
        fuz = self._fuzzy(value)
        for k, v in self._words.items():
            if v in value:
                index = value.find(v)
            else:
                v = self._fuzzy(v)
                if v in fuz:
                    index = fuz.find(v)
            if index > -1:  # 替换错误单词
                index_ = value[:index].count(' ')
                name = self._word[:index_] + k + self._word[index_ + len(k):]
                total.append(name)
                index = -1
        return total

    def _fuzzy(self, words):
        """转为模糊音"""
        total = []
        for word in words.split(' '):
            for correct, error in self.fuzzy_tone.items():
                if error not in word and correct in word:
                    word = word.replace(correct, error)
            total.append(word)
        return ' '.join(total)

    def error_word(self, word):
        """将错误的词语转为正确的词语"""
        ls = chinese_to_pin_yin(self._model, word)
        self._word = word
        total = self._flag(ls)
        return total


if __name__ == '__main__':
    ce = ChineseError(['六盘水钟山区'])
    print(ce.error_word('我在六盘谁中三区里面'))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/4/24 17:29
# @Author: Jtyoui@qq.com
from jtyoui.data import chinese_mon_number, add_time
from jtyoui.decorator import warns
import re
import datetime
import time
import itertools
import copy
import calendar


class StringTime:
    def __init__(self, sentence, date_str=None, date_format='%Y-%m-%d %H:%M:%S'):
        """传入一个字符串时间和现在时间。
        >>> st = StringTime('二零零七年十月三十一号下午2点半')
        >>> print(st.find_times())
        :param sentence: 字符串时间
        :param date_str: 你认为的现在时间，不传默认是当前时间
        :param date_format:时间格式
        """
        self._sentence = sentence
        self._localtime = date_str if date_str else time.strftime(date_format)
        self.format = date_format
        # 自定义
        self.local = time.strptime(self._localtime, self.format)
        self.re_year = r'(今年)|(明年)|(后年)|(昨年)|(前年)|(去年)|(\d+年)'
        self.re_mon = r'(上个?月)|(这个?月)|(下个?月)|(\d{0,2}本?月底?)|(\d*个?月以?后)'
        self.re_day = r'(今天)|(明天)|(后天)|(昨天)|(前天)|(\d+日)|(\d+号)|(\d*天\w?[后前])'
        self.re_week = r'(上个?周)|(下个?周)|(星期日)|(星期天)|(星期\d+)|(周\d+)'
        self.re_hour = r'(早上)|(下午)|(晚上)|(\d+点)'
        self.re_min = r'(\d+分)|(\d+点半)'
        self.re_sec = r'(\d+秒)'
        self.now_year = self.local.tm_year
        self.now_mon = self.local.tm_mon
        self.now_day = self.local.tm_mday
        self.now_week = self.local.tm_wday + 1
        self.chinese_numerals = copy.deepcopy(chinese_mon_number)
        self.chinese_numerals.pop('十')
        self.add_time = add_time
        self.times = set()

    @property
    def sentence(self):
        return self._sentence

    @sentence.setter
    def sentence(self, sentence):
        self._sentence = sentence

    def adds(self, x, fmt):
        add = datetime.datetime.strptime(self._localtime, self.format) + datetime.timedelta(days=x)
        self.now_year = add.year
        self.now_mon = add.month
        self.now_day = add.day
        self.now_week = add.isoweekday()
        return add.strftime(fmt)

    def find(self, name):
        """根据名字来查找年月日号
        ：:param name:填写：年、月、日、号、来找对应的日期
        """
        if name == '年':
            flag = '%Y'
            re_ = self.re_year
        elif name == '月':
            flag = '%m'
            re_ = self.re_mon
        elif name == '日' or name == '号':
            flag = '%d'
            re_ = self.re_day
        elif name == '周':
            flag = '%d'
            re_ = self.re_week
        else:
            flag = None
            re_ = ''
        date_time, day, add = [], 0, 0
        for d in re.findall(re_, self.sentence):
            for i in d:
                if i:
                    if i in ['星期日', '星期天']:
                        day = 7 - self.now_week
                    elif '星期' in i and i[-1].isdigit():
                        week = int(i[-1])
                        day = week - self.now_week
                    elif '周' in i and len(i) < 3:  # 周三、周四等
                        if i[-1].isdigit():
                            week = int(i[-1])
                            day = week - self.now_week
                        else:
                            add = self.add_time[i]
                    else:
                        if i in self.add_time:
                            date_time.append(self.adds(self.add_time[i], flag))
                        elif re.search(r'\d{1,2}个?月以?后', i):
                            ds = int(i[0]) if i[0].isdigit() else int(i[0:2])
                            self.now_mon = self.now_mon + ds
                        elif name in i and '底' not in i:  # 判断不是xx月底
                            if i[:-1].isdigit():
                                date_time.append(i[:-1])
                        elif '月底' in i:  # 处理xx月底
                            if i[0] == '本':
                                days = calendar.monthrange(self.now_year, self.now_mon)[1]
                                date_time.append(self.now_mon)
                                self._sentence += f'{days}日'
                            elif i[0].isdigit():
                                days = calendar.monthrange(self.now_year, int(i[0]))[1]
                                date_time.append(int(i[0]))
                                self._sentence += f'{days}日'
                            else:  # 既没有xx月也没有本月之类的。暂未考虑
                                pass
                        elif add_time.get(i[1]):
                            if i[0].isdigit():
                                date_time.append(self.adds(int(i[0]), flag))
        if day != 0 or add != 0:
            if add == 0 and date_time:
                days = int(date_time[0]) + day
                date_time = [days]
            else:
                days = self.adds(day + add, flag)
            if int(days) >= self.now_day:
                date_time.append(days)
            else:
                date_time.append(days)
                return date_time, 1
        return date_time if date_time else []

    def find_hour(self):
        """找对应的小时"""
        hours = []
        flag = 0
        for d in re.findall(self.re_hour, self.sentence):
            for i in d:
                if i:
                    if i == '早上':
                        flag = 0
                    elif i == '下午':
                        flag = 12
                    elif i == '晚上':
                        flag = 12
                    else:
                        if i[:-1].isdigit():
                            if int(i[:-1]) >= 12:
                                hours.append(int(i[:-1]))
                            else:
                                hours.append(int(i[:-1]) + flag)
                        else:
                            hours.append(0)
        return hours if hours else []

    def find_min(self):
        """找对应的分钟"""
        minute = []
        for d in re.findall(self.re_min, self.sentence):
            for i in d:
                if i:
                    if i[:-1].isdigit():
                        minute.append(int(i[:-1]))
                    elif '半' in i:
                        minute.append(30)
        return minute if minute else []

    def find_sec(self):
        """找对应的秒钟"""
        second = []
        for d in re.findall(self.re_sec, self.sentence):
            if d:
                if d[:-1].isdigit():
                    second.append(d[:-1])
        return second if second else []

    @warns('该类已经废除、废除时间2019年11月1日（19.10.28版本），请将StringTime类换成ParseTime类使用', DeprecationWarning)
    def find_times(self):
        """ 根据一句话来找对应的时间"""
        words = re.split(r'[,.，。、?!？！]', self.sentence)
        for sentences_ in words:
            if not sentences_:
                continue
            sentences = re.split(r'[到至-]', sentences_)
            t = re.findall('早上|下午|晚上', sentences[0])
            if t and len(t) == 1:
                sentences = [_ if re.findall('早上|下午|晚上', _) else t[0] + _ for _ in sentences]
            flag_y, flag_m, flag_d = [], [], []  # 临时变量，存放左右连词的性质
            for sentence in sentences:
                str_ = [self.chinese_numerals.get(s, s) for s in sentence] + [' ']  # 加[' ']的原因保证index+1不会出现list索引溢出
                string = ''
                if '十' in str_:
                    for index, c in enumerate(str_):  # 判断十在每个位置上的不同意义
                        if c == '十':
                            if str_[index - 1].isdigit() and str_[index + 1].isdigit():  # 比如：二十一实际上十可以取空，变成21
                                c = ''
                            elif str_[index - 1].isdigit() and (not str_[index + 1].isdigit()):  # 比如：二十实际上十变成0，变成20
                                c = '0'
                            elif not str_[index - 1].isdigit() and str_[index + 1].isdigit():  # 比如：十三实际上十变成1，变成13
                                c = '1'
                            else:
                                c = '10'  # 其余情况十就变成10
                        string += c
                else:
                    string = ''.join(str_)
                if re.search('[上下]个?周[1-6日]', string):
                    string = string.replace('周', '周星期')
                self._sentence = string
                y = self.find('年')  # 找到一句话中的年份
                m = self.find('月')  # 找到一句话中的月份
                d = self.find('号')  # 找到一句话中的天数
                d = d + self.find('日')  # 找到一句话中的天数
                w = self.find('周')  # 找到一句话中的天数
                if isinstance(w, tuple):
                    if m:
                        m[0] = int(m[0]) + w[1]
                    else:
                        m = [self.now_mon + w[1]]
                    d += d + w[0]
                else:
                    d += d + w
                h = self.find_hour()  # 找到一句话中的小时
                mi = self.find_min()  # 找到一句话中的分钟
                sec = self.find_sec()  # 找到一句话中的秒钟
                if not (y or m or d or h or mi or sec):
                    continue
                if not y:
                    y = flag_y
                if not m:
                    m = flag_m
                if not d:
                    d = flag_d
                if h and not d:
                    d = [self.now_day]
                flag_y, flag_m, flag_d = y, m, d
                for y_, m_, d_, h_, mi_, sec_ in itertools.zip_longest(y, m, d, h, mi, sec):
                    if not y_ and m_:
                        y_ = self.now_year
                    if not m_ and d_:
                        if not y_:
                            y_ = self.now_year
                        m_ = self.now_mon
                        add_y, add_m = divmod(m_, 12)
                        y_ += add_y
                        m_ = add_m
                    if not mi_:
                        mi_ = '00'
                    if not sec_:
                        sec_ = '00'
                    if not m_:
                        self.times.add(f'{y_}')
                    elif not d_:
                        self.times.add(f'{y_}-{m_:0>2}')
                    elif not h_:
                        self.times.add(f'{y_}-{m_:0>2}-{d_:0>2}')
                    else:
                        self.times.add(f'{y_}-{m_:0>2}-{d_:0>2} {h_:0>2}:{mi_:0>2}:{sec_:0>2}')
                    break
        times = sorted(self.times)
        self.times.clear()
        return times


if __name__ == '__main__':
    print('-----------------默认是当日期------------------')
    st = StringTime('二零零七年十月三十一号下午2点半')
    print(st.find_times())  # ['2007-10-31 14:30:00']
    st.sentence = '下周星期一下午15点半开会'
    print(st.find_times())  # ['2019-07-08 15:30:00']

    print('-----------------切换日期------------------')
    st = StringTime('下周星期一下午2点半开会', '2019-4-17 00:00:00')
    print(st.find_times())  # ['2019-04-22 14:30:00']

    print('----------------多个时间-------------------')
    st = StringTime('今天下午3点开会到4点整到12楼大会议室开会。')
    print(st.find_times())  # ['2019-07-02 15:00:00', '2019-07-02 16:00:00']

    print('----------------没有时间-------------------')
    st = StringTime('我要是不传时间呢？')
    print(st.find_times())  # []

    print('---------------只有天数--------------------')
    st = StringTime('今天去北京，明天去哪里？')
    print(st.find_times())  # ['2019-07-02', '2019-07-03']

    print('---------------跳断日期--------------------')
    st = StringTime('下周星期一下午2点半到4点开会')
    print(st.find_times())  # ['2019-07-08 14:30:00', '2019-07-08 16:00:00']

    print('---------------非常间断日期--------------------')
    st = StringTime('明天下午2点半一直到下周星期五下午4点开会')
    print(st.find_times())  # ['2019-07-03 14:30:00', '2019-07-12 16:00:00']

    print('---------------没有日期或者天数--------------------')
    st = StringTime('下午2点半开会')
    print(st.find_times())  # ['2019-07-03 14:30:00']

    print('---------------*几个月以后--------------------')
    st = StringTime('请王鹏宇下个月1号下午3点上交财务报表')
    print(st.find_times())  # ['2019-08-01 15:00:00']

    print('--------------几天之后--------------')
    st = StringTime('三天之后下午3点开会')
    print(st.find_times())  # ['2019-07-08 15:00:00']

    print('--------------几月底--------------')
    st = StringTime('明年的2月底之前必须交报告,本月底吃饭')
    print(st.find_times())  # ['2019-07-31', '2020-02-28']

    print('--------晚上-----------')
    st = StringTime('晚上11点20分')
    print(st.find_times())

    print('--------下个周几-----------')
    st = StringTime('下个周2')
    print(st.find_times())

    print('--------几个月以后的日期--------')
    st = StringTime('5个月后的明天')
    print(st.find_times())



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/14 10:36
# @Author: jtyoui@qq.com


class BrowserTypeError(Exception):
    """浏览器异常.在web模块下的ua文件"""
    ...


class UAPageVersionError(Exception):
    """ua版本异常,重新指定版本即可"""
    ...


class BaiDuMapError(Exception):
    """百度地图接口异常"""
    ...


class LibraryNotInstallError(Exception):
    """第三方库没有安装"""
    ...


class CoordinateLengthNotEqualError(Exception):
    """坐标维数不相等"""
    ...


class ParameterNotEmptyError(Exception):
    """参数不能为空"""
    ...


class MatrixNotDottedError(Exception):
    """矩阵不能点乘"""
    ...


class InconsistentLengthError(Exception):
    """长度不一致错误"""
    ...


class NameOfTrainStationNotFoundError(Exception):
    """未找到该火车站名字"""
    ...


class DownloadDataExceptionError(Exception):
    """下载数据异常"""
    ...


class NotLegitimateNumberError(Exception):
    """不是一个合法的数字"""
    ...


class NotFindPipError(Exception):
    """没有找到pip命令"""
    ...


class NumberValueError(Exception):
    """number值错误"""
    ...


class IdCardCheckError(Exception):
    """身份证校验成功"""
    ...


class NotLinuxSystemError(Exception):
    """不是Linux系统异常"""
    ...


class MathValueWarning(Warning):
    """数值错误"""
    ...


class DownLoadDataError(Exception):
    """下载数据异常"""
    ...



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/7/28 0:51
# @Author: jtyoui@qq.com
from tensorflow.examples.tutorials.mnist import input_data
from tensorflow.contrib.layers import l2_regularizer
import tensorflow as tf
from PIL import Image
import numpy as np
import os

input_node = 784  # 输入节点
output_node = 10  # 输出节点
layer_node = 500  # 隐藏层节点
regularization = 0.001  # 正则值
bitch_size = 200  # 批量大小
learning_rates = 0.1  # 初始学习率
learning_rate_decay = 0.99  # 学习衰减率
steps = 50000  # 训练步数
moving_average_decay = 0.99  # 滑动平均衰减值
model_save_path = './model/'  # 保存文件夹
model_name = 'mist_model'  # 保存名字


def get_w(shape, regular):
    """获得w权重"""
    w = tf.Variable(tf.random.truncated_normal(shape=shape, stddev=0.1))
    if regular:
        tf.compat.v1.add_to_collection(name='losses', value=l2_regularizer(regular)(w))
    return w


def get_b(shape):
    """获得b权重"""
    b = tf.Variable(tf.zeros(shape=shape))
    return b


def forward(x, regular):
    """前向传播"""
    w1 = get_w(shape=(input_node, layer_node), regular=regular)
    b1 = get_b(shape=[layer_node])
    h1 = tf.nn.relu(tf.matmul(x, w1) + b1)
    w2 = get_w(shape=(layer_node, output_node), regular=regular)
    b2 = get_b(shape=[output_node])
    y = tf.matmul(h1, w2) + b2
    return y


def get_pic(photo):
    """将照片转为输入格式"""
    img = Image.open(photo).convert('L')  # 进行转灰度
    img_array = img.resize((28, 28), Image.ANTIALIAS)  # 压缩高质量照片
    img_array = 255 - np.array(img_array)  # 将黑变白。白变黑
    img_array[img_array < 50] = 0  # 小于50转为黑色
    img_array[img_array >= 50] = 255  # 大于50转为纯白色
    nm_array = img_array.reshape((1, 784)).astype(np.float32)
    return nm_array


def backward():
    """后向传播"""
    mist = input_data.read_data_sets(train_dir='./data/', one_hot=True)
    train_data = tf.compat.v1.placeholder(tf.float32, shape=(None, input_node))
    label = tf.compat.v1.placeholder(tf.float32, shape=(None, output_node))
    y = forward(train_data, regular=regularization)
    global_step = tf.Variable(initial_value=0, trainable=False)  # 定义一个计数器，不参与训练

    # 交叉熵
    ce = tf.nn.sparse_softmax_cross_entropy_with_logits(logits=y, labels=tf.argmax(label, 1))
    cem = tf.reduce_mean(ce)

    # 正则化
    loss = cem + tf.add_n(tf.compat.v1.get_collection('losses'))

    # 验证正确率
    correct_predict = tf.equal(tf.argmax(input=y, axis=1), tf.argmax(input=label, axis=1))
    accuracy = tf.reduce_mean(tf.cast(x=correct_predict, dtype=tf.float32))

    # 指数衰减学习率
    learning_rate = tf.compat.v1.train.exponential_decay(learning_rate=learning_rates,
                                                         global_step=global_step,
                                                         decay_steps=mist.train.num_examples / bitch_size,
                                                         decay_rate=learning_rate_decay,
                                                         staircase=True)
    train_step = tf.compat.v1.train.GradientDescentOptimizer(learning_rate).minimize(loss=loss, global_step=global_step)

    # 滑动平均
    ema = tf.train.ExponentialMovingAverage(moving_average_decay, global_step)
    ema_op = ema.apply(var_list=tf.compat.v1.trainable_variables())
    with tf.control_dependencies(control_inputs=[train_step, ema_op]):
        train_op = tf.no_op(name='train')

    saver = tf.compat.v1.train.Saver()
    with tf.compat.v1.Session() as sess:
        sess.run(tf.compat.v1.global_variables_initializer())
        for i in range(steps):
            xs, ys = mist.train.next_batch(bitch_size)
            _, loss_val, step, acc = sess.run(fetches=[train_op, loss, global_step, accuracy],
                                              feed_dict={train_data: xs, label: ys})
            if i % 1000 == 0:
                print(f'loss:{loss_val},accuracy:{acc}')

        saver.save(sess, os.path.join(model_save_path, model_name), global_step=step)


def train():
    backward()


def restore_model(pic):
    with tf.Graph().as_default():
        x = tf.compat.v1.placeholder(tf.float32, shape=(None, input_node))
        y = forward(x, None)
        pre_value = tf.argmax(input=y, axis=1)

        v = tf.train.ExponentialMovingAverage(moving_average_decay).variables_to_restore()
        saves = tf.compat.v1.train.Saver(var_list=v)
        with tf.compat.v1.Session() as sess:
            cp = tf.train.get_checkpoint_state(checkpoint_dir=model_save_path)
            if cp and cp.model_checkpoint_path:
                saves.restore(sess=sess, save_path=cp.model_checkpoint_path)
                pre_value = sess.run(fetches=pre_value, feed_dict={x: pic})
                return pre_value
            else:
                print('NO Model file found!')
    return -1


def test(photo_path):
    pic = get_pic(photo_path)
    # Image.fromarray(pic.reshape((28, 28))).show()  # 展示图画数字
    return restore_model(pic / 255.0)


if __name__ == '__main__':
    # 先训练在测试
    train()  # 训练
    # value = test('./5.jpg')  # 测试
    # print(value)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time    : 2019/8/1 13:10
# @Email  : jtyoui@qq.com
# @Software: PyCharm
import time
from flask import Flask, Response, send_file, request
import gevent
from gevent.pywsgi import WSGIServer
from gevent.queue import Queue

app = Flask(__name__)
subscriptions = []


@app.route('/health/', methods=['GET'])
def health():
    return '200 OK'


@app.route('/', methods=['GET'])
def home():
    return send_file('./test.html')


class ServerSentEvent(object):

    def __init__(self, data):
        self.data = data
        self.event = None
        self.id = None
        self.desc_map = {
            self.data: "data",
            self.event: "event",
            self.id: "id"
        }

    def encode(self):
        if not self.data:
            return ""
        lines = ["%s: %s" % (v, k) for k, v in self.desc_map.items() if k]
        return "%s\n\n" % "\n".join(lines)


@app.route("/publish/epoch/end/", methods=['POST'])
def publish():
    payload = request.form.get('data')

    def notify():
        msg = str(time.time())
        for sub in subscriptions[:]:
            sub.put(payload)

    gevent.spawn(notify)
    return "OK"


@app.route("/subscribe/epoch/end/")
def subscribe():
    def gen():
        q = Queue()
        subscriptions.append(q)
        try:
            while True:
                result = q.get()
                event = ServerSentEvent(str(result))
                yield event.encode()
        except GeneratorExit:
            subscriptions.remove(q)

    return Response(gen(), mimetype="text/event-stream")


def remote_monitor(listener='localhost', port=9000):
    print(f'点击链接：http://{listener}:{port}\t')
    server = WSGIServer((listener, port), app)
    server.serve_forever()


if __name__ == '__main__':
    remote_monitor(listener='127.0.0.1')
